package edu.uoc.pfc.anonymous;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.anonymous.msg.ErrorMessages;
import edu.uoc.pfc.profile.ProfileFactory;
import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.xml.ProfileXMLHelper;
import edu.uoc.pfc.service.AnonymousQueriesClient;
import edu.uoc.pfc.social.data.SocialNetworkProfileXMLHelper;
import edu.uoc.pfc.social.exception.SocialNetworkException;
import edu.uoc.pfc.transaction.TransactionCertificateHelper;


/**
 * Programa executable amb el qual es llenaran les consultes contra el servei de 
 * consultes annimes.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class AnonymousQueriesClientStart {
	
	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(AnonymousQueriesClientStart.class);

	private static String RESULT_FILE = "./logs/queryresult.html";
	
	/**
	 * Mtode principal que s'executar per a inicialitzar el servei.
	 * 
	 * @param args [0] Fitxer amb les propietats de la base de dades de la 
	 *  			   xarxa social
	 *  		   [1] Fitxer amb les propietats del KeyStore de l'usuari.
	 *  		   [2] Port en el qual s'ha de possar en marxa el servei.
	 *  		   [3] Consulta que es vol llenar contra el servei.
	 */
	public static void main(String[] args){
		InputStream dbProperties = null;
		InputStream secProperties = null;
		FileOutputStream fos = null;
		try{
			// Obtenci dels parmetres d'entrada.
			if(args.length == 4){
				dbProperties = new FileInputStream(args[0]);
				secProperties = new FileInputStream(args[1]);
				String port = args[2];
				String query = args[3];
				
				// Obtenci de la adrea IP de l'usuari.
				InetAddress addr = InetAddress.getLocalHost();
				String ipAddress = addr.getHostAddress();
						
				// Obtenci de les dades del perfil d'usuari perfil d'usuari de la xarxa
				ProfileXMLHelper xmlHelper = new SocialNetworkProfileXMLHelper(dbProperties);
				ProfileFactory factory = 
						new ProfileFactory(xmlHelper, secProperties, ipAddress, port);
				Profile profile = factory.getProfile();
				
				// Creaci de l'objecte amb el qual es generar la signatura de la petici
				// dins del client del servei.
				TransactionCertificateHelper transHelper = 
						new TransactionCertificateHelper(profile.getUser(), profile.getContactList());
				
				// Crida al client del servei
				AnonymousQueriesClient client = new AnonymousQueriesClient(profile, transHelper);
				String answer = client.sentQueryToContact(profile.getUser(), query);
				
				// Generaci del fitxer de resultat
				File file = new File(RESULT_FILE);
				if(file.exists()){
					file.delete();
				} 
				fos = new FileOutputStream(file);
				fos.write(answer.getBytes());
				fos.flush();
				
			} else {
				LOGGER.error(ErrorMessages.USAGE_CLIENT.getValue());
			}			
		} catch (FileNotFoundException e) {
			LOGGER.error(ErrorMessages.ERROR_INPUT.getValue(), e);
		} catch (SocialNetworkException e) {
			LOGGER.error(ErrorMessages.ERROR_SOCIAL.getValue(), e);
		} catch (ProfileException e) {
			LOGGER.error(ErrorMessages.ERROR_PROFILE.getValue(), e);
		} catch (UnknownHostException e) {
			LOGGER.error(ErrorMessages.ERROR_HOST.getValue(), e);
		} catch (IOException e) {
			LOGGER.error(ErrorMessages.ERROR_QUERY_RESULT.getValue(), e);
		} finally {
			if(dbProperties != null) {
				IOUtils.closeQuietly(dbProperties);
			}
			if(secProperties!= null){
				IOUtils.closeQuietly(secProperties);
			}
			if(fos != null){
				IOUtils.closeQuietly(fos);
			}
		}
		
	}
	
}
