package edu.uoc.pfc.commons.security;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.UnrecoverableKeyException;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;

import org.apache.commons.io.IOUtils;
import org.bouncycastle.util.encoders.Base64;

import edu.uoc.pfc.commons.security.exception.CertificateHelperException;
import edu.uoc.pfc.commons.security.msg.ErrorMessages;

/**
 * Classe encarregada de manipular certificats i d'obtenir els certificats, claus pbliques, cadenes de 
 * certificaci i claus privades a partir d'un keystore.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class CertificateHelper {
	
	/**
	 * Format de certificats acceptat: X.509
	 */
	public static String CERT_FORMAT = "X.509";
	
	/**
	 * Format de keystores acceptat: PKCS#12
	 */
	public static String KEYSTORE_FORMAT = "PKCS12";
	
	/**
	 * Mtode constructor de la classe.
	 */
	public CertificateHelper(){
		
	}
	
	/**
	 * Mtode amb el qual s'obt un certificat d'un keystore a partir del seu alias.
	 * 
	 * @param keyStore Keystore on es troba el certificat que es vol obtenir.
	 * @param alias Identificador amb el qual es troba indexat el certificat dins del keystore.
	 * @return Certificat obtingut.
	 * @throws CertificateHelperException 
	 */
	public Certificate getCertificate(KeyStore keyStore, String alias) throws CertificateHelperException {
		Certificate certificate = null;       
		try {
			 if(keyStore.containsAlias(alias)){
				 certificate = keyStore.getCertificate(alias);
			 }
		} catch (KeyStoreException e) {
			throw new CertificateHelperException(ErrorMessages.
											ERROR_KEYSTORE_FORMAT.getValue(), e);	
		}    
		return certificate;
	}
	
	/**
	 * Mtode amb el qual s'obt una clau pblica d'un keystore a partir del  alias del certificat al
	 * qual pertany.
	 * 
	 * @param keyStore Keystore on es troba el certificat la clau pblica del qual que es vol obtenir.
	 * @param alias Identificador amb el qual es troba indexat el certificat dins del keystore.
	 * @return Clau pblica obtinguda.
	 * @throws CertificateHelperException 
	 */
    public PublicKey getPublicKey(KeyStore keyStore, String alias) throws CertificateHelperException {       
      PublicKey publicKey = null;
      try {
    	  if(keyStore.containsAlias(alias)){
    		  publicKey = (PublicKey)keyStore.getCertificate(alias).getPublicKey();
    	  }
  	  } catch (KeyStoreException e) {
  		  throw new CertificateHelperException(ErrorMessages.
  				  							ERROR_KEYSTORE_FORMAT.getValue(), e);	
  	  }
      return publicKey;      
    } 
	
    
    /**
     * Mtode amb el qual s'obt la cadena de certificaci d'un keystore a partir del  alias del certificat al
	 * qual pertany.
     
     * @param keyStore Keystore on es troba el certificat la cadena de certificaci del qual que es vol obtenir.
     * @param alias Identificador amb el qual es troba indexat el certificat dins del keystore.
     * @return Cadena de certificaci obtinguda.
     * @throws CertificateHelperException 
     */
    public Certificate[] getCertificatePath(KeyStore keyStore, String alias) throws CertificateHelperException {
    	Certificate[] certPath = null;
    	try {
    		if(keyStore.containsAlias(alias)){
    			certPath = keyStore.getCertificateChain(alias);
    		}
    	} catch (KeyStoreException e) {
    		throw new CertificateHelperException(ErrorMessages.
												ERROR_KEYSTORE_FORMAT.getValue(), e);
		}
    	return certPath;
    }
    
    /**
     * Mtode amb el qual d'obt una clau privada d'un keystore a partir del seu alias.
     * 
     * @param keyStore Keystore on es troba la clau privada que es vol obtenir.
     * @param alias Identificador amb el qual es troba indexada la clau privada dins del keystore.
     * @param password Contrasenya de la clau privada.
     * @return Clau privada obtinguda.
     * @throws CertificateHelperException 
     */
	public PrivateKey getPrivateKey(KeyStore keyStore, String alias, char[] password) 
																throws CertificateHelperException {
		PrivateKey privateKey = null;
		try {
			if(keyStore.isKeyEntry(alias)){
				privateKey = (PrivateKey)keyStore.getKey(alias, password);
			}
		} catch (UnrecoverableKeyException e) {
			throw new CertificateHelperException(ErrorMessages.
												ERROR_GETTING_KEY.getValue(), e);
		} catch (KeyStoreException e) {
			throw new CertificateHelperException(ErrorMessages.
												ERROR_KEYSTORE_FORMAT.getValue(), e);
		} catch (NoSuchAlgorithmException e) {
			throw new CertificateHelperException(ErrorMessages.
												ERROR_KEY_ALGORITHM.getValue(), e);	
		}
		return privateKey;       
	}
	
	/**
	 * Mtode amb el qual s'obt un objecte KeyStore a partir del InputStream del fitxer.
	 * 
	 * @param isKeyStore InputStream del Keystore.
	 * @return Objecte KeyStore.
	 * @throws CertificateHelperException 
	 */
	public KeyStore getKeystoreFromInputStream(InputStream isKeyStore, char[] password) 
															throws CertificateHelperException{
		KeyStore ks = null;
		try {
			 ks = KeyStore.getInstance(KEYSTORE_FORMAT);
			 ks.load(isKeyStore, password);			 
		} catch (FileNotFoundException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_KEYSTORE_FILE.getValue(), e);	
		} catch (KeyStoreException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_KEYSTORE_FORMAT.getValue(), e);
		} catch (NoSuchAlgorithmException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_KEY_ALGORITHM.getValue(), e);
		} catch (CertificateException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_KEYSTORE_CERTIFICATE.getValue(), e);
		} catch (IOException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_KEYSTORE_OPEN.getValue(), e);	
		}
		return ks;		
	}
	
	/**
	 * Mtode amb el qual s'obt un objecte Certificate a partir del InputStream del fitxer.
	 * 
	 * @param isCert InputStream del certificat.
	 * @return Objecte Certificate.
	 * @throws CertificateHelperException
	 */
	public Certificate getCertificateFromInputStream(InputStream isCert) 
														throws CertificateHelperException {
		Certificate cert = null;
		BufferedInputStream bis = null;
		try{
			bis = new BufferedInputStream(isCert);
	   	 	CertificateFactory cf = CertificateFactory.getInstance(CERT_FORMAT);		
	   	 	cert = cf.generateCertificate(bis);
		} catch (CertificateException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_CERTIFICATE_FORMAT.getValue(), e);	
		} finally {
			if (bis != null) {
	        	IOUtils.closeQuietly(bis);
		    }	       
		}		
		return cert;
	}
	
	/**
	 * Mtode amb el qual s'obtindr un certificat codificat en Base64.
	 * 
	 * @param cert InputStream del certificat.
	 * @return Codificaci del certificat en Base64.
	 * @throws CertificateHelperException
	 */
	public String getCertificateB64Format(Certificate cert) 
													throws CertificateHelperException{
		String certificateB64 = null;
		try{			
			byte[] b64Cert = Base64.encode(cert.getEncoded());
			certificateB64 = new String(b64Cert);
		} catch (CertificateEncodingException e) {
			throw new CertificateHelperException(ErrorMessages.
					ERROR_CERTIFICATE_FORMAT.getValue(), e);
		}
		return certificateB64;
	}
	
	/**
	 * Mtode amb el qual s'obtindr un certificat a partir de la seva codificaci 
	 * en Base64.
	 *  
	 * @param b64Cert Codificaci del certificat en Base64.
	 * @return Certificat decodificat.
	 * @throws CertificateHelperException 
	 */
	public Certificate getCertificateFromB64Format(String b64Cert) 
													throws CertificateHelperException{
		Certificate cert = null;
		InputStream is = null;
		try{
			byte[] certBytes = Base64.decode(b64Cert.getBytes());
			is = new ByteArrayInputStream(certBytes);
			CertificateFactory cf = CertificateFactory.getInstance(CERT_FORMAT);		
	   	 	cert = cf.generateCertificate(is);
		} catch (CertificateException e) {
				throw new CertificateHelperException(ErrorMessages.
						ERROR_CERTIFICATE_FORMAT.getValue(), e);	
		} finally {
			if (is != null) {
	        	IOUtils.closeQuietly(is);
		    }	       
		} 
		return cert;		
	}
	
}
