package edu.uoc.pfc.profile;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyStore;
import java.util.Properties;

import edu.uoc.pfc.commons.security.CertificateHelper;
import edu.uoc.pfc.commons.security.SecurityProperties;
import edu.uoc.pfc.commons.security.exception.CertificateHelperException;
import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.xml.ProfileXMLHelper;
import edu.uoc.pfc.profile.msg.ErrorMessages;


/**
 * Classe encarregada d'obtenir un objecte Profile amb totes les dades del perfil d'usuari i la 
 * llista de contactes emmagatzemades al fitxer local descarregat de la xarxa social, a ms 
 * del KeyStore que s'utilitzar per a signar les transaccions entre usuaris i les propietats 
 * de seguretat necessries per a poder accedir a aquest.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class ProfileFactory {
	
	/**
	 * Objecte Profile amb les dades del perfil d'usuari.
	 */
	private Profile profile;
	
	
	/**
	 * Mtode constructor amb el que es construir l'objecte Profile.
	 * 
	 * @param xmlHelper Objecte de tipus ProfileXMLHelper que ser l'encarregat de
	 * realitzar les operacions d'obtenci del Perfil actualitzat de la xarxa social
	 * o del fitxer local. 
	 * @param secProperties InputStream del fitxer de propietats de seguretat.
	 * @param ipAddress Addrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @throws ProfileException
	 */
	public ProfileFactory(ProfileXMLHelper xmlHelper, InputStream secProperties, 
							 String ipAddress, String port) throws ProfileException {
		// Inicialitzaci del perfil d'usuari.
		initializeUsersData(xmlHelper, ipAddress, port);

		// Inicialitzaci de les propietats de seguretat del perfil
		initializeSecProperties(secProperties);
	}

		
	/**
	 * Mtode amb el qual s'obt l'objecte de tipus Profile generat.
	 *  
	 * @return Objecte Profile generat.
	 */
	public Profile getProfile() {
		return profile;
	}


	/**
	 * Mtode amb el qual s'obt l'objecte Profile amb les dades actualitzades del
	 * perfil d'usuari i dels seus contactes. 
	 * 
	 * @param xmlHelper Objecte de tipus ProfileXMLHelper que ser l'encarregat de
	 * realitzar les operacions d'obtenci del Perfil actualitzat de la xarxa social
	 * o del fitxer local. 
	 * @param ipAddress Addrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @throws ProfileException
	 */
	private void initializeUsersData(ProfileXMLHelper xmlHelper, String ipAddress,
								   String port) throws ProfileException {
		File file = new File(ProfileXMLHelper.PROFILE_XML);		
		if(!file.exists()){
			 this.profile = xmlHelper.downloadProfile(ipAddress, port, file);
		} else {
			 this.profile = xmlHelper.updateProfile(ipAddress, port, file);
		}
	}
	
	/**
	 * Mtode amb el qual s'actualitzen les dades de seguretat i el KeyStore de l'usuari
	 * dins de l'objecte Profile construt. 
	 * 
	 * @param secProperties InputStream del fitxer de propietats de seguretat i del
	 * KeyStore de l'usuari.
	 * @throws ProfileException
	 */
	private void initializeSecProperties(InputStream secProperties) throws ProfileException {
		try{
			if(this.profile != null) {
				// Obtenci de les propietats del Keystore a partir del fitxer d'entrada.
				Properties secProps = new Properties();
				secProps.load(secProperties);	
				String keystore = (String) secProps.get(SecurityProperties.KEYSTORE.getValue());
				String alias = (String) secProps.get(SecurityProperties.ALIAS.getValue());
				String password = (String) secProps.get(SecurityProperties.PASSWORD.getValue());
				
				// Obtenci del Keystore a partir de les propietats del fitxer de propietats
				InputStream isKeyStore = new FileInputStream(keystore);
				CertificateHelper certHelper = new CertificateHelper();
				KeyStore keyStore = 
						certHelper.getKeystoreFromInputStream(isKeyStore, password.toCharArray());
				
				// Actualitzaci del Keystore i de les propietats d'accs dins del perfil d'usuari.
				this.profile.setKeystore(keyStore);
				this.profile.setAlias(alias);
				this.profile.setPassword(password.toCharArray());	
			}
		} catch (CertificateHelperException e) {
			throw new ProfileException(ErrorMessages.ERROR_KEYSTORE.getValue(), e);
		} catch (IOException e) {
			throw new ProfileException(ErrorMessages.ERROR_KEYSTORE.getValue(), e);
		}
	}

}
