package edu.uoc.pfc.service.rmi;

import java.io.File;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.xml.ProfileXMLHelper;
import edu.uoc.pfc.search.SearchGenerator;
import edu.uoc.pfc.search.exception.SearchGeneratorException;
import edu.uoc.pfc.service.AnonymousQueriesClient;
import edu.uoc.pfc.service.msg.ErrorMessages;
import edu.uoc.pfc.service.msg.InfoMessages;
import edu.uoc.pfc.submitting.QuerySubmittingHelper;
import edu.uoc.pfc.transaction.TransactionCertificateHelper;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;

/**
 * Implementaci del servei remot d'enviament de cerques annimes.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class AnonymousQueriesRemoteImpl extends UnicastRemoteObject 
										implements AnonymousQueriesRemote{
	

	private static final long serialVersionUID = -4108990971367352497L;

	/**
	 * Objecte que cont les dades del perfil d'usuari necessries per a
	 * a possar en marxa el protocol de cerques annimes.
	 */
	private Profile profile;
	
	/**
	 * Generador de consultes contra el motor de cerca.
	 */
	private SearchGenerator search;
	
	/**
	 * Objecte d'utilitats relacionades amb l'emmagatzemament del perfil en el fitxer XML.
	 */
	private ProfileXMLHelper xmlHelper;
	
	/**
	 * Objecte d'utilitats relacionades amb la generaci i validaci de certificats 
	 * de transacci
	 */
	private TransactionCertificateHelper transactionHelper; 
	
	/**
	 * Objecte d'utilitats relacionades amb el protocol d'enviament de consultes.
	 */
	private QuerySubmittingHelper queryHelper;
	
	/**
	 * Client del servei remot.
	 */
	private AnonymousQueriesClient client;
	
	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(AnonymousQueriesRemoteImpl.class);
		
	
	/**
	 * Mtode constructor.
	 * 
	 * @param profile Objecte que cont les dades del perfil d'usuari necessries per a
	 * a possar en marxa el protocol de cerques annimes.
	 * @param search  Generador de consultes contra el motor de cerca
	 * @param xmlHelper Objecte d'utilitats relacionades amb l'emmagatzemament del perfil 
	 * en el fitxer XML.
	 * @throws RemoteException
	 */
	public AnonymousQueriesRemoteImpl(Profile profile, SearchGenerator search, 
									  ProfileXMLHelper xmlHelper) throws RemoteException {
		super();
		this.profile = profile;
		this.search = search;
		this.xmlHelper = xmlHelper;
		
		// Inicialitzaci de les instancies de les classes de funcionalitats necessries
		this.transactionHelper = 
				new TransactionCertificateHelper(this.profile.getUser(), this.profile.getContactList());
		this.queryHelper = new QuerySubmittingHelper();
		this.client = new AnonymousQueriesClient(this.profile, this.transactionHelper);		
	}

	/**
	 * Mtode amb el qual s'envia la petici denviament duna consulta i que retorna com a 
	 * resposta el resultat de lenviament de la consulta al motor de cerca.
	 * 
	 * @param query Consulta que es vol fer al motor de cerca.
	 * @param transCert Certificat de transacci necessari per que l'usuari que 
	 * realitza la cerca quedi autentificat dins de la aplicaci.
	 * @param userSrc Identificador de l'usuari que fa la consulta.
	 * @return Resultat de la cerca en format de cadena de carcters.
	 * @throws RemoteException
	 */
	@Override
	public String getAnswer(String query, String transCert, String userSrc) 
														throws RemoteException {
		String answer = null;		
		try {
			
			LOGGER.info(InfoMessages.QUERY_RECEIVED.getValue(), 
								new String[] {userSrc, query, transCert});
			answer = processRequest(query, transCert, userSrc);
			
		} catch (SearchGeneratorException e) {
			LOGGER.error(ErrorMessages.ERROR_SEARCH.getValue(), e);
			throw new RemoteException();
		} catch (ProfileException e) {
			LOGGER.error(ErrorMessages.ERROR_PROFILE.getValue(), e);
			throw new RemoteException();
		}
		return answer;
	}
	
	/**
	 * Mtode amb el qual es processaran les peticions d'enviament de consultes concretes realitzades per 
	 * els contactes de l'usuari.
	 * 
	 * @param query Consulta que enviada per el contacte.
	 * @param transCert Certificat de transacci de la consulta.
	 * @param userSrc Identificador de l'contacte que ha delegat la consulta en l'usuari.
	 * @return Resultat de la consulta.
	 * @throws ProfileException
	 * @throws SearchGeneratorException
	 */
	private String processRequest(String query, String transCert, String userSrc) 
													throws ProfileException, SearchGeneratorException {
		String answer = null;	
		boolean accept = true;
		
		UserList contactList = profile.getContactList();
		User srcContact = contactList.get(userSrc);
		if(srcContact == null){
			LOGGER.error(ErrorMessages.ERROR_PROFILE.getValue());
			return null;
		}
						
		// Validaci del certificat de transacci que ha arribat
		boolean verify = transactionHelper.verifyTransactionCertificate(query, transCert, userSrc);
		if(verify){
			// Si la consulta ha arribat directament del propi usuari no s'avalua el nivell
			// d'egoisme perqu s tracta de la primera petici de consulta.
			if(!profile.getUser().getUserId().equals(srcContact.getUserId())){
				// Clcul del nivell d'egoisme.
				accept = queryHelper.selfishnessEvaluation(srcContact, contactList);
			}
			// Si la consulta es acceptada es guarden els valors actualitzats de l'objecte Profile 
			// i es procedeix a enviar la consulta per a obtenir la resposta.
			if(accept){	
				profile.setContactList(contactList);
				answer = sendQuery(query);
			}
		}
		File file = new File(ProfileXMLHelper.PROFILE_XML);
		xmlHelper.saveProfile(profile, file);
		return answer;
	}
	
	
	/**
	 * Mtode amb el qual s'inici el protocol d'enviament de consultes entre els
	 * contactes de l'usuari. Com a resultat retornar el resultat de la consulta. 
	 * 
	 * @param query Consulta que es vol enviar. 
	 * @return Resultat de la consulta.
	 * @throws SearchGeneratorException
	 */
	private String sendQuery(String query) throws SearchGeneratorException {
		String answer = null;
		boolean accept = false;
		
		UserList contactList = profile.getContactList();
		UserList tempContactList = contactList.getCopy();
				
		// Fins que la consulta no sigui acceptada es reintenta l'enviament. 
		while(!accept){
			// Obtenci de l'usuari en el qual es delegar l'enviament de la llista de contactes 
			// que ha de enviar la consulta.
			User destContact = queryHelper.profileExposureEstimation(tempContactList);
			
			// Si l'usuari selleccionat per a enviar la consulta s el propi usuari s'envia la consulta
			// directament cap al motor de cerca.
			if(!(profile.getUser().getUserId().equals(destContact.getUserId()))) {
				// Enviar la petici de consulta al contacte selleccionat.
				answer = client.sentQueryToContact(destContact, query);
					
				// Si no es retorna cap resposta vol dir que no s'accepta la petici
				if(answer != null){					
					accept = true;
				}
				// S'elimina l'usuari selleccionat de la llista d'usuaris disponibles per la consulta 
				// en en curs.
				tempContactList.remove(destContact);
			} else {				
				accept = true;
				answer = search.getQueryResult(query);
				LOGGER.info(InfoMessages.QUERY_SENT_SEARCH.getValue()); 
			}
			// Actualitzaci de la llista de contactes en funci del resultat de la acceptaci.
			queryHelper.updateAfterQuery(profile.getUser(), destContact, contactList, accept);
			profile.setContactList(contactList);
		}		
		return answer;
	}


}
