package edu.uoc.pfc.social.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;

import edu.uoc.pfc.social.exception.DatabaseException;
import edu.uoc.pfc.social.msg.ErrorMessages;

/**
 * Classe que ser utilitzada per a gestionar la connexi amb la base de dades. 
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class DBConnectionManager {
	
	/**
	 * Connexi amb la base de dades.
	 */
	private Connection conn = null;

	/**
	 * Mtode constructor amb el qual s'obrir la connexi amb la base de dades 
	 * en funci de les propietats de connexi que li seran pasades  per parmetre.
	 * 
	 * @param dbProps Propietats de connexi amb la base de dades.
	 * @throws DatabaseException
	 */
    public DBConnectionManager(final Properties dbProps) throws DatabaseException {
    	try{
    		String driver = (String) dbProps.get(DatabaseProperties.DRIVER.getValue());
    		String url = (String) dbProps.get(DatabaseProperties.URL.getValue());
    		String user = (String) dbProps.get(DatabaseProperties.USERNAME.getValue());
    		String password = (String) dbProps.get(DatabaseProperties.PASSWORD.getValue());

    		if ((driver == null || driver.equals("")) || (url == null || url.equals(""))
    			|| (user == null || user.equals("")) || (password == null || password.equals(""))) {
    			throw new DatabaseException(
    							ErrorMessages.ERROR_DB_PROPERTIES.getValue());
    		}
    		Class.forName(driver);
    		conn = DriverManager.getConnection(url, user, password);
    	} catch (SQLException e) {
    		throw new DatabaseException(
    							ErrorMessages.ERROR_DB_CONNECTION.getValue(), e);
		} catch (ClassNotFoundException e) {
			throw new DatabaseException(
								ErrorMessages.ERROR_DB_CONNECTION.getValue(), e);
		} 
    }

    /**
     * Method used to get the database connection.
     * 
     * @return Database connection.
     */
    public Connection getConnection() {
        return conn;
    }

    /**
     * Mtod encarregat de tencar la connexi amb la base de dades.
     * 
     * @throws DatabaseException
     */
    public void closeConnection() throws DatabaseException {
        try{
	    	if (conn != null && !(conn.isClosed())) {
	            conn.close();
	        }
        } catch(SQLException e){
        	throw new DatabaseException(
        					ErrorMessages.ERROR_DB_CLOSE.getValue(), e);
        }
    }

	/**
	 * Mtode privat encarregat de confirmar la inserci de dades en la taula.
	 * 
	 * @throws DatabaseException 
	 */
	public void acceptInsertion() throws DatabaseException {
		try {
			if(conn != null){
				conn.commit();
			}
		} catch (SQLException e) {
			throw new DatabaseException(
						ErrorMessages.ERROR_DB_INSERT.getValue(), e);
		}
	}
    
	/**
	 * Mtode privat encarregat de cancellar la inserci de dades en la taula.
	 * 
	 * @throws DatabaseException 
	 */
	public void rejectInsertion() throws DatabaseException {
		try {
			if(conn != null){
				conn.rollback();
			}
		} catch (SQLException e) {
			throw new DatabaseException(
						ErrorMessages.ERROR_DB_ROLLBACK.getValue(), e);
		}
	}
    
}
