package edu.uoc.pfc.social.db;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import edu.uoc.pfc.social.data.UserEntity;
import edu.uoc.pfc.social.exception.DatabaseException;
import edu.uoc.pfc.social.msg.ErrorMessages;

/**
 * Classe encarregada de gestionar les operacions de base de dades relacionades 
 * amb la taula Users.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class UsersDAO extends EntityDAO {

	/**
	 * Mtode constructor.
	 * 
	 * @param conn Connexi amb la base de dades.
	 */
	public UsersDAO(Connection conn) {
		super(conn);
	}
	
	/**
	 * Mtode amb el qual s'insertar una srie d'usuaris dins de la taula User
	 * 
	 * @param userList  Llista d'usuaris
	 * @throws DatabaseException 
	 */
	public void insertUserList(Map<String, UserEntity> userList) throws DatabaseException{
		PreparedStatement ps = null;
		try{
			String sql = "INSERT INTO users (id,ip_address,port,certificate) VALUES (?,?,?,?)";
			ps = getConn().prepareStatement(sql);
			
			Iterator<Entry<String, UserEntity>> it = userList.entrySet().iterator();
		    while (it.hasNext()) {
		        Map.Entry<String, UserEntity> pairs = 
		        		(Map.Entry<String, UserEntity>)it.next();
		        UserEntity user = (UserEntity) pairs.getValue();
		        ps.setString(1, user.getId());
		        ps.setString(2, user.getIpAddress());
				ps.setString(3, user.getPort());
				ps.setString(4, user.getCertificate());
				ps.executeUpdate();				
		    }			
		} catch (SQLException e) {
			throw new DatabaseException(
						ErrorMessages.ERROR_DB_INSERT_USERS.getValue(), e);
		} finally {
			closePreparedStatement(ps);
		}
	}
	
	/**
	 * Mtode utilitzat per obtenir de la base de dades les dades d'un usuari a partir 
	 * de la adrea IP i del port.
	 * 
	 * @param ipAddress Adrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @return Objecte UserEntity amb les dades de l'usuari.
	 * @throws DatabaseException
	 */
	public UserEntity getUserFromIPAddressAndPort(String ipAddress, String port) throws DatabaseException{
		UserEntity userEntity = null;
		ResultSet rs = null;
		PreparedStatement ps = null;
		String sql = "SELECT * FROM users WHERE ip_address=? AND port=?";
		
		try{
			ps = getConn().prepareStatement(sql);
			ps.setString(1, ipAddress);
			ps.setString(2, port);
			rs = ps.executeQuery();
			while (rs != null && rs.next()) {
                userEntity = getUserEntityFromResultSet(rs);
            }
		} catch(SQLException e){
			throw new DatabaseException(
					ErrorMessages.ERROR_DB_SELECT_USERS.getValue(), e);
		} finally {
			closeResultSet(rs);
			closePreparedStatement(ps);
		}
		return userEntity;
		
	}

	/**
	 * Mtode utilitzat per a obtenir de la base de dades una llista de les dades 
	 * d'usuaris a partir de la llista dels seus identificadors.
	 *    		
	 * @param userIdList Llista d'identificadors dels usuaris que es volen obtenir 
	 * de la base de dades.
	 * @return Llista d'objectes UserEntity amb les dades de tots els usuaris.
	 * @throws DatabaseException
	 */
	public List<UserEntity> getUserEntityListFromUsersIdList(List<String> userIdList) 
															throws DatabaseException{
		List<UserEntity> userEntityList = new ArrayList<UserEntity>();
		ResultSet rs = null;
		PreparedStatement ps = null;
		String sql = "SELECT * FROM users WHERE id=?";
				
		try{
			ps = getConn().prepareStatement(sql);
			for(String userId : userIdList){
				ps.setString(1, userId);
				rs = ps.executeQuery();
				if(rs != null &&  rs.next()) {
					UserEntity userEntity = getUserEntityFromResultSet(rs);
					userEntityList.add(userEntity);
				}
			}
		} catch(SQLException e){
			throw new DatabaseException(
					ErrorMessages.ERROR_DB_SELECT_USERS.getValue(), e);
		} finally {
			closeResultSet(rs);
			closePreparedStatement(ps);
		}
		return userEntityList;
	}
	
	/**
	 * Mtode privat encarregat de construr un objecte UserEntity a partir
	 * del ResultSet obtingut de la base de dades. 
	 * 
	 * @param rs ResultSet que cont els registres obtinguts de la base de dades.
	 * @return Objecte UserEntity resultant.
	 * @throws SQLException
	 */
	private UserEntity getUserEntityFromResultSet(ResultSet rs)
			throws SQLException {
		UserEntity userEntity;
		userEntity = new UserEntity();
		userEntity.setId(rs.getString("id"));
		userEntity.setIpAddress(rs.getString("ip_address"));
		userEntity.setPort(rs.getString("port"));
		userEntity.setCertificate(rs.getString("certificate"));
		return userEntity;
	}
}
