package edu.uoc.pfc.user.data;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;



/**
 * Classe que ser utilitzada per a gestionar la llista d'usuaris necessria
 * per al protocol d'enviament de consultes cap al motor de cerca.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class UserList {
	
	/**
	 * Llista d'usuaris implementada a travs d'una estructura de mapa per 
	 * a poder ser accessible a les dades de cada usuari per a per identificador
	 * de usuari.
	 */
	private Map<String, User> userList;
	
	/**
	 * Mtode constructor amb el qual s'inicialitza el mapa d'usuaris com a
	 * HashMap.
	 */
	public UserList(){
		userList = new HashMap<String, User>();
	}
	
	/**
	 * Mtode amb el qual s'obt l'usuari a partir del seu identificador 
	 * d'usuari.
	 * 
	 * @param userId Identificador d'usuari.
	 * @return Objecte User amb les dades de l'usuari de la llista associat 
	 * a l'identificador.
	 */
	public User get(String userId){
		return userList.get(userId);
	}
	
	/**
	 * Mtode amb el qual s'afegeix un nou usuari a la llista amb un 
	 * identificador concret.
	 *  
	 * @param userId Identificador d'usuari.
	 * @param user Objecte User amb les dades de l'usuari que es vol 
	 * afegir a la llista.
	 */
	public void add(String userId, User user){
		userList.put(userId, user);		
	}
	
	/**
	 * Mtode amb el qual es comprova si un usuari es troba a la llista.
	 *  
	 * @param user Objecte User amb les dades de l'usuari el qual es vol
	 * comprovar que est dins de la llista d'usuaris.
	 * @return Resultat de la comprovaci.
	 */
	public boolean contains(User user){
		return userList.containsValue(user);
	}
	
	/**
	 * Mtode amb el qual es comprova si un usuari amb un identificador 
	 * d'usuari concret es troba a la llista.
	 * 
	 * @param userId Identificador de l'usuari el qual es vol comprovar 
	 * si est dins de la llista d'usuaris.
	 * @return Resultat de la comprovaci.
	 */
	public boolean contains(String userId){
		return userList.containsKey(userId);
	}
	
	/**
	 * Mtode amb el qual s'elimina de la llista un usuari amb un 
	 * identificador d'usuari concret.
	 * 
	 * @param userId Identificador de l'usuari que es vol eliminar de la 
	 * llista.
	 */
	public void remove(String userId){
		userList.remove(userId);
	}
	
	/**
	 * Mtode amb el qual s'elimina de la llista un usuari concret.
	 * 
	 * 	@param user Identificador de l'usuari que es vol eliminar de la 
	 *  llista.
	 */
	public void remove(User user){
		userList.remove(user.getUserId());
	}
	
	/**
	 * Mtode amb el qual es comprova si la llista d'usuaris es buida.
	 * 
	 * @return Resultat de la comprovaci.
	 */
	public boolean isEmpty(){
		return userList.isEmpty();
	}
	
	/**
	 * Mtode amb el qual s'obt el nombre d'usuaris de la llista.
	 * 
	 * @return Nombre d'usuaris de la llista.
	 */
	public int size(){
		return userList.size();
	}
	
	/**
	 * Mtode amb el qual es buida la llista d'usuaris.
	 */
	public void clear(){
		userList.clear();
	}

	/**
	 * Mtode amb el qual s'obt la llista d'usuaris ordenada en funci
     * del nombre de consultes que han intentat delegar en l'usuari al 
     * qual pertany la llista.
	 * 
	 * @return Llista d'usuaris ordenada en funci del nombre de consultes 
	 * de l'usuari al qual pertany la llista ha intentat delegar en l'usuari.
	 */
	public List<User> getSortedList(){
		UserSorter sorter = new UserSorter();
		List<User> listUsers = new ArrayList<User>(userList.values());
		sorter.descSort(listUsers);
		return listUsers;
	}
	
	/**
	 * Mtode amb el qual s'obt una copia de la llista d'usuaris.
	 * @return Copia de la llista d'usuaris.
	 */
	public UserList getCopy(){
		UserList copyList = new UserList();
		for(String key : userList.keySet()){
			User copyUser = userList.get(key);
			copyList.add(key, copyUser);
		}	
		return copyList;
	}
}
