package edu.uoc.pfc.anonymous.provider;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.anonymous.msg.ErrorMessages;
import edu.uoc.pfc.profile.ProfileFactory;
import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.xml.ProfileXMLHelper;
import edu.uoc.pfc.social.data.SocialNetworkProfileXMLHelper;
import edu.uoc.pfc.social.exception.SocialNetworkException;

/**
 * Classe encarregada d'obtenir l'objecte Profile de l'usuari que executa la aplicaci en funci
 * dels parmetres d'entrada.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class ProfileProvider {

	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(ProfileProvider.class);
	
	/**
	 * Mtode constructor.
	 */
	public ProfileProvider(){
		
	}
	
	/**
	 * 
	 * Mtode amb el qual s'obtindr l'objecte encarregat de de mantenir actualitzades les dades del 
	 * fitxer XML del perfil d'usuari en funci dels parmetres d'entrada.
	 * 
	 * @param args [0] Tipus de xarxa social
	 *  		   [1] Fitxer amb les propietats del KeyStore de l'usuari.
	 *  		   [2] Port en el qual s'ha de possar en marxa el servei.
	 *  		   [3] (Opcional) Fitxer amb les propietats de la base de dades de la 
	 *  			   xarxa social.
	 * @return Objecte ProfileXMLHelper encarregat de de mantenir actualitzades les dades del 
	 * fitxer XML del perfil d'usuari. 
	 */
	public ProfileXMLHelper getProfileXMLHelper(String args[]) {
		ProfileXMLHelper xmlHelper = null;
		InputStream dbProperties = null;
		try{
			// En aquest cas noms est implementada la aplicaci per a funcionar amb la
			// xarxa social de DEMO.
			String socialNetwork = args[0].toUpperCase();
			if(socialNetwork.equals(SocialNetworkType.DEMO.toString())){
				if(args.length == 4){
					dbProperties = new FileInputStream(args[3]);
					xmlHelper = new SocialNetworkProfileXMLHelper(dbProperties);					
				} else {
					LOGGER.error(ErrorMessages.ERROR_SOCIAL_PARAMETERS.getValue(), socialNetwork, 4);					
				}
			} else {
				LOGGER.error(ErrorMessages.SOCIAL_NOT_AVAILABLE.getValue(), socialNetwork);
			}
		} catch (SocialNetworkException e) {
			LOGGER.error(ErrorMessages.ERROR_SOCIAL.getValue());
		} catch (IOException e) {
			LOGGER.error(ErrorMessages.ERROR_INPUT.getValue());
		} finally {
			if(dbProperties != null){
				IOUtils.closeQuietly(dbProperties);
			}
		}
		return xmlHelper;
	}
	
	/**
	 * 
	 * Mtode amb el qual s'obtindr l'objecte Profile amb les dades del perfil d'usuari en funci dels 
	 * parmetres d'entrada.
	 * 
	 * @param args [0] Tipus de xarxa social
	 *  		   [1] Fitxer amb les propietats del KeyStore de l'usuari.
	 *  		   [2] Port en el qual s'ha de possar en marxa el servei.
	 *  		   [3] (Opcional) Fitxer amb les propietats de la base de dades de la 
	 *  			   xarxa social.
	 * @return Objecte ProfileXMLHelper encarregat de de mantenir actualitzades les dades del 
	 * fitxer XML del perfil d'usuari. 
	 */
	public Profile getProfile(String args[], ProfileXMLHelper xmlHelper){
		Profile profile = null;
		InputStream secProperties = null;		
		try{
			if(xmlHelper != null){
				// Obtenci de la addrea IP de l'usuari.
				InetAddress addr = InetAddress.getLocalHost();
				String ipAddress = addr.getHostAddress();
				
				// Obtenci dels parmetres d'entrada.
				secProperties = new FileInputStream(args[1]);
				String port = args[2];
				
				// Obtenci de l'objecte Profile.
				ProfileFactory factory = 
						new ProfileFactory(xmlHelper, secProperties, ipAddress, port);
				profile = factory.getProfile();	
				if(profile == null){
					LOGGER.error(ErrorMessages.USER_NOT_FOUND.getValue());					
				}
			}		
		} catch (UnknownHostException e) {
			LOGGER.error(ErrorMessages.ERROR_HOST.getValue());
		} catch (ProfileException e) {
			LOGGER.error(ErrorMessages.ERROR_PROFILE.getValue());
		} catch (IOException e) {
			LOGGER.error(ErrorMessages.ERROR_INPUT.getValue());
		} finally {
			if(secProperties != null){
				IOUtils.closeQuietly(secProperties);
			}
		}
		return profile;
		
	}
	
}
