package edu.uoc.pfc.commons.security;

import java.io.IOException;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.CertificateEncodingException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.bouncycastle.cert.X509CertificateHolder;
import org.bouncycastle.cert.jcajce.JcaCertStore;
import org.bouncycastle.cert.jcajce.JcaX509CertificateConverter;
import org.bouncycastle.cms.CMSException;
import org.bouncycastle.cms.CMSProcessableByteArray;
import org.bouncycastle.cms.CMSSignedData;
import org.bouncycastle.cms.CMSSignedDataGenerator;
import org.bouncycastle.cms.CMSTypedData;
import org.bouncycastle.cms.SignerInformation;
import org.bouncycastle.cms.SignerInformationStore;
import org.bouncycastle.cms.jcajce.JcaSignerInfoGeneratorBuilder;
import org.bouncycastle.cms.jcajce.JcaSimpleSignerInfoVerifierBuilder;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.operator.ContentSigner;
import org.bouncycastle.operator.OperatorCreationException;
import org.bouncycastle.operator.jcajce.JcaContentSignerBuilder;
import org.bouncycastle.operator.jcajce.JcaDigestCalculatorProviderBuilder;
import org.bouncycastle.util.Store;

import edu.uoc.pfc.commons.security.exception.SignatureHelperException;
import edu.uoc.pfc.commons.security.msg.ErrorMessages;


/**
 * Classe encarregada de signar i validar signatures electrniques en format CMS. 
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class SignatureHelper {
	
	public static String BC_PROVIDER = "BC";
	
	public static String SIGNATURE_ALG = "SHA1withRSA";
	
	
	/**
	 * Mtode constructor de la classe que inicialitza el provedor criptogrfic.
	 */
	public SignatureHelper(){
		Security.addProvider(new BouncyCastleProvider());
	}
	
	
	/**
	 * Mtode amb el qual es generar la signatura electrnica d'unes dades en format CMS. 
	 * 
	 * @param data Conjunt de bytes que es volen signar electrnicament.
	 * @param cert Certificat del propietari de la clau pblica.
	 * @param privKey Clau privada amb la qual es generar la signatura electrnica.
	 * @return Resultat de calcular la signatura electrnica.
	 * @throws SignatureHelperException 
	 */
	public byte[] sign(byte[] data, Certificate cert, PrivateKey privKey) throws SignatureHelperException {
		byte[] signature = null;
		
		try{						
			CMSTypedData dataToSig = new CMSProcessableByteArray(data);
			X509Certificate x509Cert = (X509Certificate)cert;
			List<X509Certificate> certList = new ArrayList<X509Certificate>();
			certList.add(x509Cert);
			Store certs = new JcaCertStore(certList);
			
			ContentSigner sha1Signer = new JcaContentSignerBuilder(SIGNATURE_ALG)
											.setProvider(BC_PROVIDER).build(privKey);
			
			CMSSignedDataGenerator generator = new CMSSignedDataGenerator();			
			generator.addSignerInfoGenerator(
					new JcaSignerInfoGeneratorBuilder(
							new JcaDigestCalculatorProviderBuilder()
							.setProvider(BC_PROVIDER).build()).build(sha1Signer, x509Cert));
			generator.addCertificates(certs);
			
			CMSSignedData sigData = generator.generate(dataToSig, true);			
			signature = sigData.getEncoded();
			
		} catch (CertificateEncodingException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_SIGNATURE_CERT.getValue(), e);
		} catch (OperatorCreationException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_SETTING_SIGNER.getValue(), e);
		} catch (CMSException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_GENERATING_SIGNATURE.getValue(), e);
		} catch (IOException e) {			
			throw new SignatureHelperException(ErrorMessages.ERROR_GETTING_BYTES.getValue(), e);
		}  		
		return signature;
	}
	
	
	/**
	 * Mtode amb el qual es validar una signatura electrnica en format CMS retornant com a 
	 * resultat les dades que han estat signades.
	 * 
	 * @param signature Signatura electrnica que es vol validar.
	 * @param cert Certificat amb el qual s'ha generat la firma i amb el qual ser validada.
	 * @return Dades que han estat signades.
	 * @throws SignatureHelperException 
	 */
	public byte[] verify(byte[] signature, Certificate cert) throws SignatureHelperException {
		byte[] signedData = null;
		
		try{			
			CMSSignedData sigData = new CMSSignedData(signature);	
			SignerInformationStore signers = sigData.getSignerInfos();
			Store certStore = sigData.getCertificates();
			
			Collection<?> sigCollection = signers.getSigners();
			Iterator<?> sigIt = sigCollection.iterator();
			while (sigIt.hasNext()) {
				SignerInformation signer = (SignerInformation)sigIt.next();
				Collection<?> certCollection = certStore.getMatches(signer.getSID());
				Iterator<?>certIt = certCollection.iterator();
		        X509CertificateHolder certHolder = (X509CertificateHolder)certIt.next();
		        X509Certificate certSig = new JcaX509CertificateConverter()
		        						.setProvider(BC_PROVIDER).getCertificate(certHolder);
		        boolean certFound = certSig.equals((X509Certificate)cert);
		        boolean sigValid = signer.verify(new JcaSimpleSignerInfoVerifierBuilder()
									.setProvider(BC_PROVIDER).build(certHolder));
				if (sigValid && certFound) {
					CMSTypedData signedContent = (CMSTypedData)sigData.getSignedContent() ;
					signedData = (byte[])signedContent.getContent();
					break;
				}
			}	
			
		} catch (CertificateException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_SIGNATURE_CERT.getValue(), e);
		} catch (OperatorCreationException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_GETTING_SIGNER.getValue(), e);
		} catch (CMSException e) {
			throw new SignatureHelperException(ErrorMessages.ERROR_VALIDATING_SIGNATURE.getValue(), e);
		} 		
		return signedData;
	}

}
