package edu.uoc.pfc.commons.xml;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Classe encarregada d'extreure informaci continguda dins d'un document XML
 * i d'oferir funcionalitats per crear nous documents XML a partir de nous valors.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class XMLParser {
	
	/**
	 * Objecte amb el qual s'obtindr la estructura d'un document XML.
	 */
	private DocumentBuilder docBuilder;
	
	/**
	 * Objecte amb el qual s'obtindran elements del document XML a travs 
	 * de expressions XPath.
	 */
	private XPath xPath;

	/**
	 * Mtode constructor.
	 * 
	 * @throws ParserConfigurationException 
	 */
	public XMLParser() throws ParserConfigurationException{
		XPathFactory factory = XPathFactory.newInstance();
		this.xPath = factory.newXPath();
		DocumentBuilderFactory docFactory = 
			 			DocumentBuilderFactory.newInstance();
		this.docBuilder = docFactory.newDocumentBuilder();		
	}

	/**
	 * Mtode a travs del cual s'obtindr la estructura d'un document XML 
	 * a partir del Input Stream del document.
	 * 
	 * @param is InputStream del document XML.
	 * @return Estructura Document del document XML.
	 * @throws IOException 
	 * @throws SAXException 
	 */
	public Document getDocumentFromInputStream(InputStream is) 
										throws SAXException, IOException {
		Document doc =  docBuilder.parse(new InputSource(is));		
		return doc;
	}
	
	/**
	 * Mtode a travs del cual s'obtindr una llista de nodes que acompleixi una
	 * expressi XPath a partir del node arrel del qual penja la llista de nodes que
	 * es vol obtenir.
	 * 
	 * @param node Node arrel del qual penja la llista de nodes que volem obtenir.
	 * @param xpathExpr Expressi XPath utilitzada per a obtenir la llista de nodes.
	 * @return Estructura NodeList amb la llista de nodes.
	 * @throws XPathExpressionException 
	 */
	public NodeList getNodeListFormExpression(Node node, String xpathExpr) 
												throws XPathExpressionException{
		XPathExpression xPathIdxExpr = xPath.compile(xpathExpr);
		NodeList nodeList = 
				(NodeList)xPathIdxExpr.evaluate(node, XPathConstants.NODESET);
		return nodeList;		
	}
	
	/**
	 * Mtode a travs del cual s'obtindr un node concret que acompleixi una
	 * expressi XPath a partir del node arrel del qual penja el nodes que es 
	 * vol obtenir.
	 * 
	 * @param node Node arrel del qual penja el nodes que es vol obtenir.
	 * @param xpathExpr Expressi XPath utilitzada per a obtenir el node.
	 * @return Estructura Node amb el node que volem obtenir.
	 * @throws XPathExpressionException 
	 */
	public Node getNodeFromExpression(Node node, String xpathExpr) 
											throws XPathExpressionException{
		XPathExpression xPathIdxExpr = xPath.compile(xpathExpr);
		Node nodeResult = (Node)xPathIdxExpr.evaluate(node, XPathConstants.NODE);		 
		return nodeResult;
	}
	
	/**
	 * Mtode a travs del cual s'obtindr el node arrel d'un document XML.
	 * 
	 * @param doc Document XML.
	 * @return Node arrel del document XML.
	 */
	public Element getRootNode(Document doc) {
		Element root = doc.getDocumentElement();
		return root;
	}
	
	/**
	 * Mtode a travs del qual es crea un nou document XML.
	 * 
	 * @return Objecte Document amb la estructura del nou document.
	 */
	public Document createDocument(){
		DOMImplementation impl = docBuilder.getDOMImplementation();
		Document doc = impl.createDocument(null,null,null);
        return doc;
	}
	
	/**
	 * Mtode encarregat de crear un nou node.
	 * 
	 * @param doc Estructura del document XML.
	 * @param nodeName Nom del node que es vol crear
	 * @return Node creat.
	 */
	public Element createNode(Document doc, String nodeName){
		Element elem = doc.createElement(nodeName);
		return elem;
	}
	
	/**
	 * Mtode encarregat d'afegir el node arrel al document XML.
	 * 
	 * @param doc Estructura del document XML a la qual es vol afegir
	 * el node arrel.
	 * @param rootNode Node arrel del document
	 */
	public void addRootNode(Document doc, Element rootNode){
		doc.appendChild(rootNode);
	}
	
	/**
	 * Mtode encarregat d'afegir un nou subnode a un altre node.
	 * 
	 * @param parentNode Node sota el qual es volen afegir nous nodes
	 * @param subNode Node que ha de penjar del node pare.
	 */
	public void addSubnode(Element parentNode, Element subNode){
		parentNode.appendChild(subNode);
	}
	
	/**
	 * Mtode encarregat d'afegir un atribut a un node.
	 * 
	 * @param node Node al qual es vol afegir un atribut.
	 * @param attribute Nom de l'atribut que es vol afegir al node.
	 * @param value Valor de l'atribut.
	 */
	public void addAttribute(Element node, String attribute, String value){
		node.setAttribute(attribute, value);
	}
	
	/**
	 * Mtode encarregat d'assignar un contingut textual a un node.
	 * 
	 * @param node Node al qual es vol assignar un valor textual.
	 * @param content Valor textual a afegir al node.
	 */
	public void addTextContext(Element node, String content){
		node.setTextContent(content);
	}
		
	
	/**
	 * Mtode utilitat per a exportar un document XML a un OuputStream.
	 * 
	 * @param doc Document XML que es vol exportar a OutputStream
 	 * @param os OutputStream del fitxer on s'escriur el document XML.
	 * @throws TransformerException 
	 */
	public void writeDocument(Document doc, OutputStream os) throws TransformerException{
		DOMSource domSource = new DOMSource(doc);
        TransformerFactory tf = TransformerFactory.newInstance();
        Transformer transformer = tf.newTransformer();
        transformer.setOutputProperty(OutputKeys.METHOD, "xml");
        transformer.setOutputProperty(OutputKeys.ENCODING,"UTF-8");
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        StreamResult sr = new StreamResult(os);
        transformer.transform(domSource, sr);
	}
}
