package edu.uoc.pfc.profile.xml;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.cert.Certificate;
import java.util.List;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.xpath.XPathExpressionException;

import org.apache.commons.io.IOUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import edu.uoc.pfc.commons.security.CertificateHelper;
import edu.uoc.pfc.commons.security.exception.CertificateHelperException;
import edu.uoc.pfc.commons.xml.XMLParser;
import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.msg.ErrorMessages;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;

/**
 * 
 * Classe encarregada de passar de objecte Profile a document XML i de
 * document XML a objecte Profile.
 * 
 * @author Rubn Salvador Gareta 
 *
 */
public class ProfileXMLParser {
	

	/**
	 * Objecte amb el qual s'accedir al contingut de l'XML.
	 */
	private XMLParser parser;
		
	
	/**
	 * Mtode constructor.
	 * 
	 * @throws ProfileException 
	 */
	public ProfileXMLParser() throws ProfileException{
		try{
			this.parser = new XMLParser();
		} catch (ParserConfigurationException e) {
			throw new ProfileException(ErrorMessages.ERROR_XML_CONFIG.getValue(), e);
		}
	}
	
	
	/**
	 * Mtode amb el qual, a partir d'un objecte Profile s'obt un OutputStream
	 * amb el document XML.
	 * 
	 * @param profile Objecte amb les dades del perfil d'usuari.
	 * @param file Fitxer on s'escriur l'XML obtingut.
	 * @throws ProfileException 
	 */
	public void profileToXMLFile(Profile profile, File file) throws ProfileException{
		FileOutputStream os = null;
		try{
			// Creaci del fitxer de sortida.			
			os = new FileOutputStream(file);
			
			// Obtenci les dades de l'usuari i la seva llista de contactes
			// de l'objecte Profile.
			User user = profile.getUser();
			UserList contactList = profile.getContactList();			
			if(user != null && contactList != null){
				// Creaci del document XML corresponent al perfil d'usuari.
				Document doc = parser.createDocument();
				Element userNode = createUserNode(doc, user);
				parser.addRootNode(doc, userNode);
				Element contactsNode = createContactsNode(doc, contactList);
				parser.addSubnode(userNode, contactsNode);
				
				// Copia del document XML en el OutputStream del fitxer.
				parser.writeDocument(doc, os);
			}		
		} catch (TransformerException e) {
			throw new ProfileException(ErrorMessages.ERROR_XML_PROFILE.getValue(), e);
		} catch (CertificateHelperException e) {
			throw new ProfileException(ErrorMessages. ERROR_XML_PROFILE.getValue(), e);
		} catch (FileNotFoundException e) {
			throw new ProfileException(ErrorMessages. ERROR_XML_PROFILE.getValue(), e);
		} finally {
			if(os!= null){
				IOUtils.closeQuietly(os);
			}
		}		
	}
	
	/**
	 * Mtode encarregat d'obtenir un objecte Profile amb les dades del perfil de
	 * l'usuari a partir d'un fitxer XML.
	 *  
	 * @param file Fitxer XML del qual es vol obtenir l'objecte Profile. 
	 * @return Objecte Profile amb les dades del perfil de l'usuari.
	 * @throws ProfileException 
	 */
	public Profile profileFromXMLFile(File file) throws ProfileException{
		InputStream is = null;
		Profile profile = null;
		try{
			// Obtenci de la estructura del document XML a partir del fitxer.
			is = new FileInputStream (file);
			Document profileXML = parser.getDocumentFromInputStream(is);
						
			// Obtenci del node arrel del document amb les dades de l'usuari.
			Element userNode = parser.getRootNode(profileXML);
			
			// Obtenci de les dades de l'usuari.
			User user = getUserFromUserNode(userNode);
			
			// Obtenci de la llista de contactes de l'usuari.
			UserList contactList = getUserListFromUserNode(userNode);
			
			// Construcci de l'objecte profile. 
			profile = new Profile();
			profile.setUser(user);
			profile.setContactList(contactList);
			
		} catch (FileNotFoundException e) {
			throw new ProfileException(ErrorMessages.ERROR_XML_PROFILE.getValue(), e);
		} catch (SAXException e) {
			throw new ProfileException(ErrorMessages.ERROR_XML_PROFILE.getValue(), e);
		} catch (IOException e) {
			throw new ProfileException(ErrorMessages.ERROR_XML_PROFILE.getValue(), e);
		} catch (XPathExpressionException e) {
			throw new ProfileException(ErrorMessages.ERROR_SEARCH_ELEMENT.getValue(), e);
		} catch (CertificateHelperException e) {
			throw new ProfileException(ErrorMessages. ERROR_XML_PROFILE.getValue(), e);
		} finally {
			if(is!= null){
				IOUtils.closeQuietly(is);
			}
		}		
		return profile;
	}


	/**
	 * Mtode encarregat de crear la llista de contactes a partir d'una llista 
	 * d'usuaris. 
	 * 
	 * @param doc Estructura del document XML on es vol crear la llista de contactes.
	 * @param contactList Llista de contactes amb les dades dels usuaris.
	 * @throws CertificateHelperException 
	 */
	private Element createContactsNode(Document doc, UserList contactList) 
												throws CertificateHelperException{
		Element contactsNode = parser.createNode(doc, ProfileXMLTags.CONTACTLIST_NODE.getValue());
		List<User> userList = contactList.getSortedList();
		for(User user : userList) {
			Element userNode = createUserNode(doc, user);
			parser.addSubnode(contactsNode, userNode);
		}
		return contactsNode;
	}

	
	/**
	 * Mtode encarregat de generar un node amb les dades d'un usuari.
	 * 
	 * @param doc Estructura del document XML on es vol crear el node User.
	 * @param user Node user amb les dades de l'usuari.
	 * @return Node User.
	 * @throws CertificateHelperException 
	 */
	private Element createUserNode(Document doc, User user) 
												throws CertificateHelperException {
		Element userNode = parser.createNode(doc, ProfileXMLTags.USER_NODE.getValue());	
		parser.addAttribute(userNode, ProfileXMLTags.ID.getValue(), user.getUserId());
		parser.addAttribute(userNode, ProfileXMLTags.T.getValue(), Long.toString(user.getT()));
		parser.addAttribute(userNode, ProfileXMLTags.TAU.getValue(), Long.toString(user.getTau()));
		parser.addAttribute(userNode, ProfileXMLTags.P.getValue(), Double.toString(user.getP()));
			
		Element certNode = parser.createNode(doc, ProfileXMLTags.CERTIFICATE_NODE.getValue());
		CertificateHelper certHelper = new CertificateHelper();
		String certStr = certHelper.getCertificateB64Format(user.getCert());
		parser.addTextContext(certNode, certStr);
		parser.addSubnode(userNode, certNode);
		
		return userNode;
	}
	
	/**
	 * Mtode encarregat de construir un objecte de tipus User amb 
	 * tota la informaci emmagatzemada en un node del document XML 
	 * del perfil d'usuari. 
	 * 
	 * @param userNode Node amb la informaci de l'usuari.
	 * @return Objecte User amb les dades de l'usuari 
	 * @throws XPathExpressionException 
	 * @throws CertificateHelperException 
	 */
	private User getUserFromUserNode(Element userNode) 
												throws XPathExpressionException, 
													   CertificateHelperException {
		User user = new User();
		user.setUserId(userNode.getAttribute(ProfileXMLTags.ID.getValue()));
		user.setT(Long.parseLong(userNode.getAttribute(ProfileXMLTags.T.getValue())));
		user.setTau(Long.parseLong(userNode.getAttribute(ProfileXMLTags.TAU.getValue())));
		user.setP(Double.parseDouble(userNode.getAttribute(ProfileXMLTags.P.getValue())));
		
		Element certNode = 
					(Element)parser.getNodeFromExpression(userNode, 
									ProfileXPathExpr.CERTIFICATE.getValue());
		String b64Cert = certNode.getTextContent();
		CertificateHelper certHelper = new CertificateHelper();
		Certificate cert = certHelper.getCertificateFromB64Format(b64Cert);
		user.setCert(cert);
		
		return user;
	}
	
	/**
	 * Mtode encarregat de construir un objecte de tipus UserList amb
	 * la llista de contactes de l'usuari.
	 *  
	 * @param userNode Node amb la informaci de l'usuari.
	 * @return Objecte UserList amb la llista de contactes de l'usuari.
	 * @throws XPathExpressionException 
	 * @throws CertificateHelperException 
	 */
	private UserList getUserListFromUserNode(Element userNode) 
												throws XPathExpressionException, 
													   CertificateHelperException {
		UserList contactList = new UserList();
		// Obtenci del node arrel de la llista de contactes de l'usuari.
		Element contactListNode = 
				(Element)parser.getNodeFromExpression(userNode, 
								ProfileXPathExpr.CONTACT_LIST.getValue());
				
		// Obtenci de la llista de nodes dels contactes de l'usuari.
		NodeList contactNodeList = 
				parser.getNodeListFormExpression(contactListNode, 
								ProfileXPathExpr.PROFILE_USER.getValue());
		
		// Obtenci de la llista d'objectes User a partir de la llista 
		// de nodes
		for (int j = 0; j < contactNodeList.getLength(); j++) {
			Element contactNode = (Element)contactNodeList.item(j);
			User user = getUserFromUserNode(contactNode);
			contactList.add(user.getUserId(), user);			
		}
		return contactList;
	}
	
}
