package edu.uoc.pfc.social;

import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.util.Map;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.social.data.UserEntity;
import edu.uoc.pfc.social.db.ContactsDAO;
import edu.uoc.pfc.social.db.DBConnectionManager;
import edu.uoc.pfc.social.db.UsersDAO;
import edu.uoc.pfc.social.exception.DatabaseException;
import edu.uoc.pfc.social.exception.SocialNetworkException;
import edu.uoc.pfc.social.exception.SocialNetworkXMLParseException;
import edu.uoc.pfc.social.msg.ErrorMessages;
import edu.uoc.pfc.social.msg.InfoMessages;
import edu.uoc.pfc.social.xml.SocialNetworkXMLParser;

/**
 * Clase encarregada d'inicialitzar la xarxa social a partir d'un fitxer XML amb
 * la informaci dels usuaris i les propietats de la base de dades.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class SocialNetwork {
	
	/**
	 * Representaci de la xarxa social en format XML
	 */
	private InputStream socialNetwork;
	
	/**
	 * Objecte amb el qual es gestionar la connexi amb la abase de dades.
	 */
	private DBConnectionManager dbConnMng;
	
	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(SocialNetwork.class);
	
	/**
	 * Mtode constructor. 
	 * 
	 * @param socialNetwork InputStream del fitxer XML que cont la 
	 * representaci de la xarxa social.
	 * @param dbPropsIs InputStream del fitxer de properties que cont les
	 * propietats de la base de dades.
	 * @throws SocialNetworkException 
	 */
	public SocialNetwork(InputStream socialNetwork, InputStream dbPropsIs)
												throws SocialNetworkException {
		try{
			LOGGER.info(InfoMessages.SOCIAL_START.getValue());
			this.socialNetwork = socialNetwork;
			initDBConnectionManager(dbPropsIs);
		} catch (IOException e) {
			LOGGER.error(ErrorMessages.ERROR_INPUT_FILES.getValue(), e);
			throw new SocialNetworkException(
								ErrorMessages.ERROR_INPUT_FILES.getValue(), e);
		} catch (DatabaseException e) {
			LOGGER.error(ErrorMessages.ERROR_DATABASE_CONNECT.getValue(), e);
			throw new SocialNetworkException(
								ErrorMessages.ERROR_DATABASE_CONNECT.getValue(), e);
		}
	}

	
	/**
	 * Mtode amb el qual s'inicialitzar la xarxa social amb les dades contingudes al
	 * fixer.
	 * @throws SocialNetworkException 
	 */
	public void init() throws SocialNetworkException{
		try{
			LOGGER.info(InfoMessages.SOCIAL_LOADING.getValue());
			// Obtenci de la estructura d'usuaris de la xarxa social
			SocialNetworkXMLParser parser = new SocialNetworkXMLParser();	
			Map<String, UserEntity> userList = parser.parseXMLFile(socialNetwork);
			
			//Emmagatzemament de la estructura en la base de dades
			storeUsers(userList);			
			LOGGER.info(InfoMessages.SOCIAL_FINISH.getValue());
		} catch (SocialNetworkXMLParseException e) {
			LOGGER.error(ErrorMessages.ERROR_INPUT_FILES.getValue(), e);
			throw new SocialNetworkException(
								ErrorMessages.ERROR_INPUT_FILES.getValue(), e);
		} catch (DatabaseException e) {
			LOGGER.error(ErrorMessages.ERROR_DATABASE_INSERT.getValue(), e);
			throw new SocialNetworkException(
								ErrorMessages.ERROR_DATABASE_INSERT.getValue(), e);
		} 
	}
	
	/**
	 * Mtode amb el cual es tanca la connexi amb la xarxa social.
	 * 
	 * @throws SocialNetworkException 
	 */
	public void close() throws SocialNetworkException{
		try{
			dbConnMng.closeConnection();
		} catch (DatabaseException e) {
			LOGGER.error(ErrorMessages.ERROR_DATABASE_DISCONNECT.getValue(), e);
			throw new SocialNetworkException(
							ErrorMessages.ERROR_DATABASE_DISCONNECT.getValue(), e);
		} 
	}

	/**
	 * Mtode privat utilitzat per emmagatzemar els usuaris i els contactes en la
	 * base de dades.
	 *  
	 * @param dbPropsIs InputStream amb les propietats de la base de dades.
	 * @throws IOException
	 * @throws DatabaseException
	 */
	private void initDBConnectionManager(InputStream dbPropsIs)
											throws IOException, DatabaseException {
		Properties dbProperties = new Properties();
		dbProperties.load(dbPropsIs);
		dbConnMng = new DBConnectionManager(dbProperties);		
	}
	
	
	/**
	 * Mtode privat utilitzat per emmagatzemar els usuaris i els contactes en la
	 * base de dades. 
	 * 
	 * @param userList Estructura amb la llista d'usuaris.
	 * @throws DatabaseException 
	 */
	private void storeUsers(Map<String, UserEntity> userList)
											throws DatabaseException {
		try{			
			Connection conn = dbConnMng.getConnection();			
			UsersDAO usersDAO = new UsersDAO(conn);
			ContactsDAO contactsDAO = new ContactsDAO(conn);			
			usersDAO.insertUserList(userList);
			contactsDAO.insertContacts(userList);
			dbConnMng.acceptInsertion();			
		} catch(DatabaseException e){
			dbConnMng.rejectInsertion();
			throw e;
		}
	}

}
