package edu.uoc.pfc.social.data;

import java.io.IOException;
import java.io.InputStream;
import java.security.cert.Certificate;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import edu.uoc.pfc.commons.security.CertificateHelper;
import edu.uoc.pfc.commons.security.exception.CertificateHelperException;
import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.social.db.ContactsDAO;
import edu.uoc.pfc.social.db.DBConnectionManager;
import edu.uoc.pfc.social.db.UsersDAO;
import edu.uoc.pfc.social.exception.DatabaseException;
import edu.uoc.pfc.social.exception.SocialNetworkException;
import edu.uoc.pfc.social.msg.ErrorMessages;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;

/**
 * Classe encarregada de realitzar la transformaci entre les dades dels usuaris que 
 * venen de la base de dades de la xarxa social y les dades necessaries per el
 * protocol de consultes annimes.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class SocialNetworkProfileMapper {
		
	/**
	 * Representaci de la xarxa social en format XML
	 */
	private DBConnectionManager dbConnMng;
	

	/**
	 * Mtode constructor.
	 * 
	 * @param dbProperties InputStream del fitxer de les propietats de connexi de
	 * la base de dades. 
	 * @throws SocialNetworkException
	 */
	public SocialNetworkProfileMapper(InputStream dbProperties) throws SocialNetworkException{
		try {
			initDBConnectionManager(dbProperties);
		} catch (IOException e) {
			throw new SocialNetworkException(ErrorMessages.ERROR_INPUT_FILES.getValue(), e);
		} catch (DatabaseException e) {
			throw new SocialNetworkException(ErrorMessages.ERROR_DATABASE_CONNECT.getValue(), e);
		}
	}
	
	/**
	 * Mtode utilitzat per a obtenir un objecte del tipus Profile necessari per a
	 * treballar dins del protocol de consultes annimes a partir de la addrea IP
	 * i el port de connexi de l'usuari.
	 * 
	 * @param ipAddress Adrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @return Objecte Profile amb les dades de l'usuari necessries per al protocol
	 * de consultes annimes.
	 * @throws SocialNetworkException 
	 */
	public Profile getProfileFromUserAddress(String ipAddress, String port) 
											throws SocialNetworkException{
		Profile profile = null;		
		try {
			Connection conn = dbConnMng.getConnection();
			UsersDAO usersDAO = new UsersDAO(conn);
			ContactsDAO contactsDAO = new ContactsDAO(conn);
			
			// Obtenci de les dades de l'usuari i dels seus contactes a partir de
			// la adrea IP i del port de connexi.
			UserEntity userEntity = getUserEntityFromIPAndPort(ipAddress, port, 
															   usersDAO, contactsDAO);
			User user = getUserFromUserEntity(userEntity);
			
			// Obtenci de les dades dels contactes de l'usuari a partir de la llista 
			// d'identificadors.
			List<UserEntity> userEntityList = 
					getUserEntityContactListFromUserEntity(userEntity, usersDAO);
			UserList contactList = getUserListFromUserEntityList(userEntityList); 
			
			//Construcci de l'objecte Profile.
			profile = generateProfile(user, contactList);
			
		} catch (DatabaseException e){
			throw new SocialNetworkException(ErrorMessages.ERROR_DATABASE_SELECT.getValue(), e);
		} finally {
			closeDBConnection();
		}
		return profile;		
	}


	/**
	 * Mtode privat encarregat d'inicialitzar el gestor de connexi amb la 
	 * base de dades. 
	 * 
	 * @param dbPropsIs InputStream del fitxer de les propietats de connexi de
	 * la base de dades. 
	 * @throws DatabaseException
	 * @throws IOException 
	 */
	private void initDBConnectionManager(InputStream dbPropsIs) 
										throws DatabaseException, IOException {
		Properties dbProperties = new Properties();		
		try{
			dbProperties.load(dbPropsIs);
			dbConnMng = new DBConnectionManager(dbProperties);
		} catch (DatabaseException e) {			
			dbConnMng.closeConnection();
			throw e;
		} 		
	}
	
	/**
	 * Mtode utilitzat per a tancar la connexi amb la base de dades. 
	 * 
	 * @throws SocialNetworkException
	 */
	private void closeDBConnection() throws SocialNetworkException {
		try{
			dbConnMng.closeConnection();
		} catch (DatabaseException e){
			throw new SocialNetworkException(ErrorMessages.ERROR_DATABASE_DISCONNECT.getValue(), e);
		}
	}
	
	/**
	 * Mtode encarregat de recuperar de la base de dades el objecte UserEntity
	 * amb les dades de l'usuari i la llista de identificadors dels seus contactes
	 * a partir de la seva adrea IP i del seu port de connexi. 
	 * 
	 * @param ipAddress Adrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @param usersDAO DAO de la taula Users necessari per a obtenir les dades
	 * de l'usuari.
	 * @param contactsDAO DAO de la taula Contacts necessari per a obtenir la
	 * llista de identificadors dels contactes de l'usuari.
	 * @return Objecte UserEntity amb les dades de l'usuari.
	 * @throws DatabaseException
	 */
	private UserEntity getUserEntityFromIPAndPort(String ipAddress, String port,
									 UsersDAO usersDAO, ContactsDAO contactsDAO)  
											 			throws DatabaseException {
		UserEntity userEntity = null;		
		try{
			userEntity = usersDAO.getUserFromIPAddressAndPort(ipAddress, port);
			if(userEntity != null){
				List<String> contactIdList = 
							contactsDAO.getContactsIdListByUserId(userEntity.getId());
				userEntity.setContactIdList(contactIdList);
			}
		} catch (DatabaseException e) {			
			dbConnMng.closeConnection();
			throw e;
		}	
		return userEntity;
	}
	
	
	/**
	 * Mtode utilitzat per a obtenir una llista d'objectes UserEntity amb les dades dels 
	 * contactes d'un usuari a partir del objecte UserEntity amb les dades del usuari.
	 * 
	 * @param userEntity Objecte UserEntity amb les dades del usuari del qual es volen obtenir
	 * les ades dels seus contactes.
	 * @param usersDAO DAO de la taula Users necessari per a obtenir les dades
	 * de l'usuari.
	 * @return Llista d'objectes UserEntity amb les dades dels contactes de l'usuari.
	 * @throws DatabaseException 
	 */
	private List<UserEntity> getUserEntityContactListFromUserEntity(UserEntity userEntity, 
												 UsersDAO usersDAO) throws DatabaseException {
		List<UserEntity> userEntityList = null;
		try{
			if(userEntity != null){
				List<String> contactsIdList = userEntity.getContactIdList();
				userEntityList = new ArrayList<UserEntity>();
				userEntityList = usersDAO.getUserEntityListFromUsersIdList(contactsIdList);
			}
		} catch (DatabaseException e) {			
			dbConnMng.closeConnection();
			throw e;
		}
		return userEntityList;		
	}
	
	/**
	 * Mtode encarregat d'obtenir un objecte User amb les dades de l'usuari en el
	 * format necessari per el protocol de consultes annimes.
	 * 
	 * @param userEntity Objecte UserEntity que cont les dades de l'usuari obtingudes
	 * de la base de dades de la xarxa social.
	 * @return Objecte User que cont les dades de l'usuari necessries per al protocol
	 * de seguretat.
	 * @throws SocialNetworkException 
	 */
	private User getUserFromUserEntity(UserEntity userEntity) throws SocialNetworkException{
		User user = null;
		CertificateHelper certHelper = new CertificateHelper();
		try {
			if(userEntity != null){
				// Obtenci de les dades del User a partir de les dades del UserEntity.
				user = new User();
				Certificate cert = certHelper
						.getCertificateFromB64Format(userEntity.getCertificate());
				user.setUserId(userEntity.getIpAddress()
						.concat(":").concat(userEntity.getPort()));
				user.setCert(cert);
				user.setP(1.0);
				user.setT(0);
				user.setTau(0);
			}			
		} catch (CertificateHelperException e) {
			throw new SocialNetworkException(ErrorMessages.ERROR_STORED_DATA.getValue(), e);
		}
		return user;		
	}
	
	/**
	 * Mtode utilitzat per a obtenir un objecte UserList amb la llista de contactes
	 * a partir de la llista d'objectes UserEntity corresponent amb la llista de 
	 * contactes de l'usuari amb dades extretes directament de la base de dades.
	 * 
	 * @param userEntityList Llista d'objectes UserEntity amb les dades dels contactes
	 * de l'usuari.
	 * @return Objecte UserList amb la llista de contactes de l'usuari.
	 * @throws SocialNetworkException 
	 */
	private UserList getUserListFromUserEntityList(List<UserEntity> userEntityList) 
															throws SocialNetworkException {
		UserList userList = null;
		if(userEntityList != null){
			userList = new UserList();
			for(UserEntity userEntity : userEntityList){
				User user = getUserFromUserEntity(userEntity);
				userList.add(user.getUserId(), user);
			}
		}		
		return userList;		
	}
	
	/**
	 * Mtode encarregat de construir lobjecte Profile amb les dades de l'usuari i la 
	 * llista dels seus contactes.
	 * 
	 * @param user Dades de l'usuari del qual es vol construir el seu objecte profile.
	 * @param contactList Llista de contactes de l'usuari.
	 * @return Objecte Profile amb les dades de l'usuari i la seva llista de contactes.
	 */
	private Profile generateProfile(User user, UserList contactList) {
		Profile profile = null;
		if(user!=null && contactList != null){
			// El propi usuari tamb s'afegeix a la llista UserList perqu a ell tamb
			// se li poden enviar consultes.
			profile = new Profile();
			profile.setUser(user);
			contactList.add(user.getUserId(), user);
			profile.setContactList(contactList);
		}
		return profile;
	}

}
