package edu.uoc.pfc.social.data;

import java.io.File;
import java.io.InputStream;
import java.util.List;

import edu.uoc.pfc.profile.data.Profile;
import edu.uoc.pfc.profile.exception.ProfileException;
import edu.uoc.pfc.profile.xml.ProfileXMLHelper;
import edu.uoc.pfc.profile.xml.ProfileXMLParser;
import edu.uoc.pfc.social.exception.SocialNetworkException;
import edu.uoc.pfc.social.msg.ErrorMessages;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;

/**
 * Classe encarregada de mantenir actualitzat el fitxer corresponent al perfil 
 * de l'usuari dins del protocol de cerques annimes. Es tracta d'una implementaci
 * de la interface ProfileXMLHelper adaptada a la xarxa social desenvolupada en
 * aquest projecte.
 *   
 * @author Rubn Salvador Gareta
 *
 */
public class SocialNetworkProfileXMLHelper extends ProfileXMLHelper {
	
	/**
	 * Objecte necessari per a obtenir les dades del perfil d'usuari
	 */
	private SocialNetworkProfileMapper mapper;
	
	
	/**
	 * Mtode constructor.
	 * 
	 * @param dbProperties InputStream del fitxer de propietats de connexi 
	 * a la base de dades.
	 * @throws SocialNetworkException 
	 */
	public SocialNetworkProfileXMLHelper(InputStream dbProperties) throws SocialNetworkException{
		mapper = new SocialNetworkProfileMapper(dbProperties);		
	}
	
	
	/**
	 *
	 * Mtode encarregat d'obtenir un fitxer XML amb les dades.
	 * 
	 * @param ipAddress Adrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @param file Fitxer on s'escriur l'XML obtingut.
	 * @return Objecte Profile amb les dades del perfil d'usuari.
	 * @throws ProfileException
	 */
	public Profile downloadProfile(String ipAddress, String port, File file) 
													throws ProfileException{
		Profile profile = null;
		try{
			profile = mapper.getProfileFromUserAddress(ipAddress, port);
			if(profile != null) {
				ProfileXMLParser parser = new ProfileXMLParser();
				parser.profileToXMLFile(profile, file);
			}
		} catch (SocialNetworkException e) {
			throw new ProfileException(ErrorMessages.ERROR_PROFILE.getValue(), e);
		} 
		return profile;
	}
	

	/**
	 * Mtode encarregat d'actualitzar el fitxer XML amb les noves dades.
	 * 
	 * @param ipAddress  Adrea IP de l'usuari.
	 * @param port Port de connexi de l'usuari.
	 * @param file Fitxer que cont l'XML que s'ha d'actualitzar.
	 * @throws ProfileException
	 */
	public Profile updateProfile(String ipAddress, String port, File file) 
														throws ProfileException{
		Profile profileFromSocial = null;
		try{
			profileFromSocial = mapper.getProfileFromUserAddress(ipAddress, port);
			if(profileFromSocial != null){
				ProfileXMLParser parser = new ProfileXMLParser();
				Profile profileFromFile = parser.profileFromXMLFile(file);				
				updateUserAndContacts(profileFromFile, profileFromSocial);
				parser.profileToXMLFile(profileFromSocial, file);
			}
		} catch (SocialNetworkException e) {
			throw new ProfileException(ErrorMessages.ERROR_PROFILE.getValue(), e);
		}
		return profileFromSocial;
	}
	
	/**
	 * Mtode amb el qual s'actualitzen els atributs de l'usuari i de la llista 
	 * de contactes de l'usuari descarregada de la xarxa social, amb 
	 * els valors acumulats durant l's continu del protocol.
	 * 
	 * @param profileFromFile Objecte Profile amb les dades de l'usuari
	 * i dels seus contactes acumulades.
	 * @param profileFromSocial Objecte Profile amb les dades de l'usuari
	 * i dels seus contactes descarregades de la xarxa social.
	 */
	private void updateUserAndContacts(Profile profileFromFile, Profile profileFromSocial){
		User userFromFile = profileFromFile.getUser();
		User userFromSocial = profileFromSocial.getUser();		
		UserList contactListFile = profileFromFile.getContactList(); 
		UserList contactListSocial = profileFromSocial.getContactList();
		updateUser(userFromFile, userFromSocial);
		updateUserList(contactListFile, contactListSocial);
		profileFromSocial.setUser(userFromSocial);
		profileFromSocial.setContactList(contactListSocial);
	}
	
	/**
	 * Mtode amb qual s'actualizen els valors de l'usuari obtingut de la
	 * xarxa social amb els que es tenien emmagatzemats al fitxer. 
	 * 
	 * @param userFromFile Dades de l'usuari obtingudes del fitxer.
	 * @param userFromSocial Dades de l'usuari obtingudes de la xarxa
	 * social.
	 */
	private void updateUser(User userFromFile, User userFromSocial){
		userFromSocial.setT(userFromFile.getT());
		userFromSocial.setTau(userFromFile.getTau());
		userFromSocial.setP(userFromFile.getP());
	}
		
	/**
	 * Mtode amb qual s'actualizen els valors dels usuaris de la llista
	 * d'usuaris obtinguda de la xarxa social amb els que es tenien 
	 * emmagatzemats al fitxer. 
	 * 
	 * @param userListFromFile Dades dels usuaris de la llista de contactes
	 * de l'usuari obtingudes del fitxer.
	 * @param userListFromSocial Dades dels usuaris de la llista de contactes
	 * de l'usuari obtingudes de la xarxa social.
	 */
	private void updateUserList(UserList userListFromFile, 
								UserList userListFromSocial){
		List<User> userSocialList = userListFromSocial.getSortedList();
		for (User userFromSocial : userSocialList){
			User userFromFile = userListFromFile.get(userFromSocial.getUserId());
			if(userFromFile != null){
				updateUser(userFromFile, userFromSocial);
				userListFromSocial.add(userFromSocial.getUserId(), userFromSocial);
			}
		}
	}

}
