package edu.uoc.pfc.social.xml;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import edu.uoc.pfc.commons.xml.XMLParser;
import edu.uoc.pfc.social.msg.ErrorMessages;
import edu.uoc.pfc.social.data.UserEntity;
import edu.uoc.pfc.social.exception.SocialNetworkXMLParseException;

/**
 * Classe encarregada de llegir un fitxer XML on s la configuraci de la xarxa 
 * social i obtenir la estructura de dades necessria per a inserir les dades dels
 * usuaris a la base de dades.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class SocialNetworkXMLParser {
	
	/**
	 * Objecte amb el qual s'accedir al contingut de l'XML.
	 */
	private XMLParser parser;
	
	/**
	 * Mtode constructor.
	 * 
	 * @throws SocialNetworkXMLParseException 
	 */
	public SocialNetworkXMLParser() throws SocialNetworkXMLParseException{
		try{
			this.parser = new XMLParser();
		} catch (ParserConfigurationException e) {
			throw new SocialNetworkXMLParseException(
					ErrorMessages.ERROR_XML_CONFIG.getValue(), e);
		}
	}
	
	/**
	 * Mtode amb el qual, a partir el InputStream de un document XML s'obt una
	 * estructura de mapa amb les dades del usuari que s'han demmagatzemar a la 
	 * base de dades.
	 * 
	 * @param is InputStream amb el document XML que cont la informaci de la
	 * xarxa social.
	 * @return Estructura de mapa amb tots els usuaris que han d'emmagatzemar-se
	 * a la base de dades.
	 * @throws SocialNetworkXMLParseException 
	 */
	public Map<String, UserEntity> parseXMLFile(InputStream is) 
												throws SocialNetworkXMLParseException{
		Map<String,UserEntity> userList = new HashMap<String, UserEntity>();
		try{	
			// Obtenci de la estructura del document XML a partir del fitxer fent
			// servir les expressions XPath.
			Document socialNetworkDoc = parser.getDocumentFromInputStream(is);
			NodeList userNodeList = 
					parser.getNodeListFormExpression(socialNetworkDoc.getDocumentElement(), 
										SocialNetworkXPathExpr.SOCIAL_USER.getValue());
			
			for (int i = 0; i < userNodeList.getLength(); i++) {		   
				// Obtenci dels elements del document XML necessaris per a generar 
				// la estructura de dades dels usuaris.
				Element userNode = (Element)userNodeList.item(i);
							   
				// Generaci de la estructura de dades de l'usuari i els seus contactes.
				UserEntity userEntity = generateUserEntity(userNode);
			   
				// Inserci de la estructira de dades del usuari a la taula de dades
				// que ser utilitzada per a insertar la informaci a la base de dades.
				userList.put(userEntity.getId(), userEntity);
			}
		} catch (XPathExpressionException e) {
			throw new SocialNetworkXMLParseException(
									ErrorMessages.ERROR_SEARCH_ELEMENT.getValue(), e);
		} catch (SAXException e) {
			throw new SocialNetworkXMLParseException(
									ErrorMessages.ERROR_XML_DOCUMENT.getValue(), e);
		} catch (IOException e) {
			throw new SocialNetworkXMLParseException(
									ErrorMessages.ERROR_XML_DOCUMENT.getValue(), e);
		} 
		return userList;		
	}

	/**
	 * Mtode privat amb el qual s'obtindr un objecte de la classe UserEntity amb 
	 * tota la informaci de l'usuari corresponent al node que es passa com a parmetre.
	 * 
	 * @param userNode Node del document XML corrcorresponent a un usuari.
	 * @return Objecte UserEntity amb tota la informaci de l'usuari que es vol 
	 * insertar a la base de dades.
	 * @throws XPathExpressionException
	 */
	private UserEntity generateUserEntity(Element userNode) throws XPathExpressionException {
		// Inicialitzaci de les dades de l'usuari.
		UserEntity userEntity = getUserData(userNode);
		
		// Inicialitzaci de la llista de contactes de l'usuari.
		List<String> contactIdList = getContactIdList(userNode);
		userEntity.setContactIdList(contactIdList);
		return userEntity;
	}

	/**
	 * Mtode privat amb el qual es crea un objecte UserEntity amb tota la informaci
	 * de l'usuari que haur de ser emmagatzemat a la base de dades.
	 * 
	 * @param userNode Node del document XML que cont la informaci de l'usuari.
	 * @return Objecte UserEntity amb les dades de l'usuari.
	 * @throws XPathExpressionException 
	 */
	private UserEntity getUserData(Element userNode) throws XPathExpressionException {
		// Inicialitzaci de tota la informaci a partir dels atributs del document XML.
		UserEntity userEntity = new UserEntity();
		userEntity.setId(userNode.getAttribute(SocialNetworkXMLTags.ID.getValue()));
		userEntity.setIpAddress(userNode.getAttribute(
										SocialNetworkXMLTags.IP_ADDRESS.getValue()));
		userEntity.setPort(userNode.getAttribute(
										SocialNetworkXMLTags.PORT.getValue()));
		
		// Obtenci del node corresponen al certificat de l'usuari.
		Element certNode = 
				(Element)parser.getNodeFromExpression(userNode, 
								SocialNetworkXPathExpr.CERTIFICATE.getValue());
		userEntity.setCertificate(certNode.getTextContent());		
		return userEntity;
	}
	
	/**
	 * Mtode privat amb el qual s'obt la llista de identificadors dels contactes
	 * de l'usuari.
	 * 
	 * @param userNode Node del document XML que cont la informaci de l'usuari.
	 * @return Llista d'identificadors dels contactes de l'usuari.
	 * @throws XPathExpressionException
	 */
	private List<String> getContactIdList(Element userNode)
			throws XPathExpressionException {
		// Obtenci del node arrel de la llista de contactes de l'usuari.
		Element contactListNode = 
				(Element)parser.getNodeFromExpression(userNode, 
								SocialNetworkXPathExpr.CONTACT_LIST.getValue());
		
		// Obtenci de la llista de nodes dels contactes de l'usuari.
		NodeList contactNodeList = 
				parser.getNodeListFormExpression(contactListNode, 
								SocialNetworkXPathExpr.CONTACT.getValue());
		
		// Construcci de la llista d'identificadors dels contactes de
		// l'usuari.
		List<String> contactIdList = new ArrayList<String>();
		for (int j = 0; j < contactNodeList.getLength(); j++) {
			Element contactNode = (Element)contactNodeList.item(j);
			contactIdList.add(contactNode.getAttribute(SocialNetworkXMLTags.ID.getValue()));			
		}
		return contactIdList;
	}

}
