package edu.uoc.pfc.submitting;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.submitting.msg.ErrorMessages;
import edu.uoc.pfc.submitting.msg.InfoMessages;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;

/**
 * Classe mitjanant la qual s'obtindran les estimacions necessaries per a decidir 
 * a quin usuari de la llista de contactes s'ha d'enviar una consulta i per a decidir 
 * si una consulta que arriba procedent d'un usuari ha de ser acceptada o no.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class QuerySubmittingHelper {
	
	/**
	 * Constant utilitzada per a penalitzar els usuaris del sistema que
	 * tenen comportaments egoistes.
	 */
	public static final double GAMMA = 0.02; 
	
	
	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(QuerySubmittingHelper.class);
	
	/**
	 *  Mtode constructor.
	 * 
	 */
	public QuerySubmittingHelper(){
	
	}
	
	/**
	 * Mtode que s'encarrega de calcullar la funci d'estimaci del nivell
	 * d'exposici del perfil dels usuaris.
	 * 
	 * @param contacts Llista d'usuaris que cont tots els contactes de 
	 * l'usuari i el propi usuari.
	 * @return Usuari de la llista al qual li ser envia la consulta.
	 */
	public User profileExposureEstimation(UserList contacts){
		User estimatedUser = null;
		double alpha = Double.MAX_VALUE;
		int beta = 1;
		LOGGER.info(InfoMessages.START_PROFILE_EXPOSURE.getValue());
		
		// Iteraci sobre la llista de contactes per a determinar l'usuari
		// al qual li ser enviada la consulta.
		if(!contacts.isEmpty()){
			//Obtenim la llista ordenada en funci del nombre de consultes 
			//que el contacte ha intentat delegar en l'usuari. 
			List<User> userList = contacts.getSortedList();
			for(int i = 0; i < contacts.size(); i++){
				User currentUser = userList.get(i);
				double currentAlpha = ((double)currentUser.getTau())/((double)beta);
				// El contacte que tingui el valor d'alpha ms petit (nombre estimat de
				// consultes generades per l'usuari que el contacte ha enviat cap el motor 
				// de cerca ms petit).
				if(currentAlpha < alpha){
					alpha = currentAlpha;
					estimatedUser = currentUser;				
				}
				beta++;				
			}
			LOGGER.info(InfoMessages.USER_SEND.getValue() + estimatedUser.getUserId());
		} else {
			LOGGER.error(ErrorMessages.ERROR_CONTACT_LIST.getValue());
		}
		return estimatedUser;
	}
	
	/**
	 * Mtode que s'encarrega d'avaluar el nivell d'egoisme dels usuaris i 
	 * d'actualitzar els percentatges de acceptaci
	 * 
	 * @param userSrc Contacte que ha enviat la consulta a l'usuari i del 
	 * qual es vol avaluar el seu nivell d'egoisme. 
	 * @param contacts Llista de tots contactes de l'usuari.
	 * @return Valor de la avaluaci del nivell d'egoisme.
	 */
	public boolean selfishnessEvaluation(User userSrc, UserList contacts){
		boolean acceptQuery = false;
		LOGGER.info(InfoMessages.START_SELFISHNESS_EVAL.getValue());
		
		// S'obt el resultat de la avaluaci i es recalculen els percentatges.
	    if(contacts.contains(userSrc)){
			double random = Math.random();
		    if(random <= userSrc.getP()){
		    	acceptQuery = true;		    	
		    	userSrc.setP(userSrc.getP() - GAMMA);		    	
		    	LOGGER.info(InfoMessages.QUERY_ACCEPTED.getValue(), userSrc.getUserId());		    	
		    } else {
		    	acceptQuery = false;		    	
		    	LOGGER.info(InfoMessages.QUERY_REJECTED.getValue(), userSrc.getUserId());
		    }
			
			// Independentemnt del resultat de la avaluaci, s'incrementa el nombre de 
		    // consultes que el contacte ha intentat delegar en l'usuari.
			userSrc.setT(userSrc.getT() + 1);
			
			// S'actualitza l'usuari de la llista.
			contacts.add(userSrc.getUserId(), userSrc);
		
	    } else {
	    	LOGGER.error(ErrorMessages.ERROR_CONTACT.getValue(), userSrc.getUserId());	    	
	    }
		return acceptQuery;		
	}
	
	/**
	 * Mtode que s'encarrega d'actualitzar la probabilitat de l'usuari d'acceptar consultes d'un
	 * dels seus contactes en funci del restultat de una petici d'enviament realitzada prviament
	 * i de mantenir el nombre de consultes que el contacte ha acceptat de l'usuari.
	 * 
	 * @param userSrc Usuari que ha enviat la consulta. 
	 * @param userDst Contacte del qual es vol recalcular la probabilitat d'acceptar les seves 
	 * consultes. 
	 * @param contacts Llista d'usuaris.
	 * @param accept Resultat de l'avaluaci del nivell d'egoisme.
	 */
	public void updateAfterQuery(User userSrc, User userDst, UserList contacts, boolean accept){
		// Si el contacte triat per a enviar la consulta s el mateix usuari, s'han d'actualitzar
		// el comptador de consultes delegades en l'usuari per part del mateix usuari
		if(userSrc.getUserId().equals(userDst.getUserId())){
			userSrc.setT(userSrc.getT() + 1);				
		}
		if(accept){			
			userDst.setTau(userDst.getTau() + 1);	
			userDst.setP(userDst.getP() + (2 * GAMMA));
		} else {
			userDst.setP(userDst.getP() - GAMMA);			
		}
		contacts.add(userDst.getUserId(), userDst);
	}

}
