package edu.uoc.pfc.transaction;

import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.util.Calendar;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import edu.uoc.pfc.commons.security.CertificateHelper;
import edu.uoc.pfc.commons.security.SignatureHelper;
import edu.uoc.pfc.commons.security.exception.CertificateHelperException;
import edu.uoc.pfc.commons.security.exception.SignatureHelperException;
import edu.uoc.pfc.transaction.data.TransactionCertificate;
import edu.uoc.pfc.transaction.data.TransactionCertificateSerializer;
import edu.uoc.pfc.transaction.msg.ErrorMessages;
import edu.uoc.pfc.transaction.msg.InfoMessages;
import edu.uoc.pfc.user.data.User;
import edu.uoc.pfc.user.data.UserList;


/**
 * 
 * Classe mitjaant la qual es generar i es validar la prova o certificat de transacci 
 * que acompanyar sempre una consulta quan aquesta sigui enviada d'un usuari a un altre.
 * 
 * @author Rubn Salvador Gareta
 *
 */
public class TransactionCertificateHelper {
	
	/**
	 * Usuari que crear la instncia de la classe.
	 */
	private User user;
	
	/**
	 * Llista d'usuaris que sn contactes directes de l'usuari.
	 */
	private UserList userList;
	
	
	/**
	 * Variable que ser utilitzada per a escriure missatges en el log de la aplicaci.
	 */
	private static final Logger LOGGER =
	        LoggerFactory.getLogger(TransactionCertificateHelper.class);
	
	/**
	 * Mtode constructor de la classe.
	 * 
	 * @param user Usuari que crear la instncia de la classe.
	 * @param userList Llista de contactes directes de l'usuari.
	 */
	public TransactionCertificateHelper(User user, UserList userList){
		this.user = user;
		this.userList = userList;		
	}


	/**
	 * Mtode amb el qual es crea la prova o certificat de transacci entre dos usuaris.
	 * 
	 * @param q Consulta que l'usuari vol enviar a un altre usuari.
	 * @param userDst Identificador de l'usuari al qual es vol enviar la consulta.
	 * @param keyStore Objecte KeyStore on es troba la clau privada i el certificat per a 
	 * signar les dades del certificat de transacci.
	 * @param alias Identificador per accedir a la clau privada i al certificat dins del
	 * Keystore.
	 * @param password Contrasenya de la clau privada.
	 * @return Certificat o prova de transacci entre dos usuaris codificat en Base 64.
	 */
	public String generateTransactionCertificate(String q, String userDst, KeyStore keyStore, 
												 String alias, char[] password) {
		
		LOGGER.info(InfoMessages.START_TRANS_CERT_GEN.getValue());
		
		// Comprobaci de que els parmetres son correctes.
		if(!checkGenerateParameters(q, userDst)){
			return null;
		}
				
		// Instanciaci de les clases de funcionalitats que seran utilitzades
		TransactionCertificateSerializer serializer = new TransactionCertificateSerializer();
		SignatureHelper sigHelper = new SignatureHelper();
		CertificateHelper certHelper = new CertificateHelper();
		try {
			// Obtenci del certificat i la clau privada per generar la signatura.
			Certificate cert = certHelper.getCertificate(keyStore, alias);
			PrivateKey privKey = certHelper.getPrivateKey(keyStore, alias, password);
			
			//Inicialitzaci del objecte amb la informaci sobre la transacci
			byte[] serialTransCert = generateTransactionCertificate(q, userDst, serializer);
			LOGGER.info(InfoMessages.TRANS_CERT.getValue() + new String(serialTransCert));
			
			// Generaci de la signatura electrnica del objecte que cont la informaci 
			// sobre la transacci serialitzat.
			byte[] signature = sigHelper.sign(serialTransCert, cert, privKey);
			
			// Codificaci de la signatura en Base64.
			String transCert = new String(serializer.encode(signature));
			LOGGER.info(InfoMessages.OK_SIGNATURE.getValue() + transCert);			
			return transCert;
			
		} catch (CertificateHelperException e) {
			LOGGER.error(ErrorMessages.ERROR_KEYSTORE.getValue(), e);
			return null;
		}  catch (SignatureHelperException e) {
			LOGGER.error(ErrorMessages.ERROR_SIGNATURE.getValue(), e);
			return null;		
		}		
	}


	/**
	 * Mtode amb el qual es valida que la prova o certificat de transacci entre dos usuaris es correcta.
	 * 
	 * @param q Consulta rebuda per l'usuari que ha estat enviada per un altre usuari.
	 * @param transCert Certificat de transacci codificat en Base64 enviat conjuntament amb la consulta.
	 * @param userSrc Identificador de l'usuari que envia la consulta.
	 * @return Resultat de la validaci del certificat de transacci.
	 */
	public boolean verifyTransactionCertificate(String q, String transCert, String userSrc) {
		
		LOGGER.info(InfoMessages.START_TRANS_CERT_VAL.getValue());
		
		// Comprobaci de que els parmetres son correctes.
		if(!checkVerifyParameters(q, transCert, userSrc)){
			return false;
		}
				
		// Instanciaci de les clases de funcionalitats que seran utilitzades
		TransactionCertificateSerializer serializer = new TransactionCertificateSerializer();
		SignatureHelper sigHelper = new SignatureHelper();
				
		// Obtenci de la signatura electrnica a partir del certificat de transacci rebut.
		byte[] signature = serializer.decode(transCert.getBytes());
		
		// Validaci de la signatura i obtenci de les dades que han estat signades i que es
		// troben contingundes dins de la prpia signatura.
		// Obtenci del certificat electrnic de l'usuari que envia el certificat de transacci.
		User userSrcObj = userList.get(userSrc);
		Certificate cert = userSrcObj.getCert();
		byte[] tcSerialBytes;
		try {
			tcSerialBytes = sigHelper.verify(signature, cert);
		} catch (SignatureHelperException e) {
			LOGGER.error(ErrorMessages.ERROR_SIG_VALIDATION.getValue(), e);
			tcSerialBytes = null;
		}
		
		// Si la signatura cont les dades que han estat signades, es valida que aquestes
		// tingin els valors adequats per a que el certificat de transacci sigui considerat
		// correcte.
		if(tcSerialBytes != null){
			// Recuperaci del objecte a partir de les dades serialitzades.
			TransactionCertificate tc = serializer.deserialize(tcSerialBytes);
			
			// Validaci de que les dades son correctes.			
			if(validateTransactionCertificate(q, userSrc, tc)){
				LOGGER.info(InfoMessages.OK_VALIDATION.getValue());
				return true;
			}
		}
		LOGGER.error(ErrorMessages.ERROR_DATA.getValue());
		return false;				
	}


	/**
	 * Mtode amb el qual, a partir dels valors dels seus camps, es genera un 
	 * transaction certificate serialitzat.
	 * 
	 * @param q Consulta que l'usuari vol enviar a un altre usuari.
	 * @param userDst Identificador de l'usuari al qual es vol enviar la consulta.
	 * @param serializer Instancia de l'objecte que realitza les operacions de
	 * serielitzat del objecte amb les dades.
	 * @return
	 */
	private byte[] generateTransactionCertificate(String q, String userDst,
			TransactionCertificateSerializer serializer) {
		TransactionCertificate tc = new TransactionCertificate();
		tc.setQ(q);
		tc.setUserSrc(this.user.getUserId());
		tc.setUserDst(userDst);
		long timeStamp = Calendar.getInstance().getTimeInMillis();
		tc.setTimestamp(String.valueOf(timeStamp));
		return serializer.serialize(tc);		
	}
	
	/**
	 * Mtode amb el qual es valida que les dades del certificat de transacci son 
	 * coherents.
	 * 
	 * @param q Consulta rebuda per l'usuari que ha estat enviada per un altre usuari.
	 * @param userSrc Identificador de l'usuari que envia la consulta.
	 * @param tc Objecte amb les dades que cont el certificat de transacci que 
	 * es vol validar
	 * @return Resultat de la validaci.
	 */
	private boolean validateTransactionCertificate(String q, String userSrc, 
											TransactionCertificate tc){
		boolean validity = true;
		long timeStamp = Calendar.getInstance().getTimeInMillis();

		// Validaci de que la consulta enviada s la mateixa que la consulta signada. 
		if(!tc.getQ().equals(q)){
			LOGGER.error(ErrorMessages.ERROR_Q.getValue());
			validity = false;
		}
		
		// Validaci de que el destinatari de la consulta s el mateix que l'usuari 
		// que la ha rebut. 
		if(!tc.getUserDst().equals(user.getUserId())){
			LOGGER.error(ErrorMessages.ERROR_USER_DST.getValue());
			validity = false;
		}
		
		// Validaci de que l'usuari que envia la consulta s el mateix que l'usuari 
		// que ha generat el certificat de transacci. 
		if(!tc.getUserSrc().equals(userSrc)){
			LOGGER.error(ErrorMessages.ERROR_USER_SRC.getValue());
			validity = false;
		}
		
		// Validaci de que el moment de la generaci de la signatura s anterior que
		// el moment actual.
		if(!(Long.parseLong(tc.getTimestamp()) < timeStamp)) {
			LOGGER.error(ErrorMessages.ERROR_TIMESTAMP.getValue());
			validity = false;
		}		
		return validity;
		
	}	
	
	/**
	 * Mtode utilitzat per comprobar que els parmentres d'entrada de la funci
	 * de generaci de certificat de transacci sn correctes.
	 * 
	 * @param q Consulta que l'usuari vol enviar a un altre usuari.
	 * @param userDst Objecte KeyStore on es troba la clau privada i el certificat per a 
	 * signar les dades del certificat de transacci.
	 * @return Resultat de la validaci.
	 */
	private boolean checkGenerateParameters(String q, String userDst) {
		// Si la consulta o el usuari dest estan buits, el certificat de 
		// transacci no es podr generar.
		boolean parametersOK = true;
		if(q == null){
			LOGGER.error(ErrorMessages.ERROR_QUERY_GEN.getValue());
			parametersOK = false;
		}
		if(userDst == null){
			LOGGER.error(ErrorMessages.ERROR_USERDEST.getValue());
			parametersOK = false;
		}
		return parametersOK;
	}
	
	/**
	 * Mtode utilitzat per comprobar que els parmentres d'entrada de la funci
	 * de validaci de certificat de transacci sn correctes.
	 * 
	 * @param q onsulta rebuda per l'usuari que ha estat enviada per un altre usuari.
	 * @param transCert Certificat de transacci codificat en Base64 enviat conjuntament amb la consulta.
	 * @param userSrc Identificador de l'usuari que envia la consulta.
	 * @return  Resultat de la validaci.
	 */
	private boolean checkVerifyParameters(String q, String transCert,
			String userSrc) {
		// Si la consulta, el certificat de transacci o el usuari que ha enviat la consulta estan buits, 
		// el certificat de transacci no es podr validar.
		boolean parametersOK = true;
		if(q == null){
			LOGGER.error(ErrorMessages.ERROR_TRANSACTION.getValue());
			parametersOK = false;
		}
		if(transCert == null){
			LOGGER.error(ErrorMessages.ERROR_TRANSACTION.getValue());
			parametersOK = false;
		}
		if(userSrc == null){
			LOGGER.error(ErrorMessages.ERROR_USER.getValue());
			parametersOK = false;
		}
		// Si l'usuari no es troba a la llista d'usuaris i el certificat de transacci no ha estat enviat
		// o si no es disposa del certificat electrnic de l'usuari es considerar que el certificat de 
		// transacci no es correcte.				
		User userSrcObj = userList.get(userSrc);
		if(userSrcObj == null){
			LOGGER.error(ErrorMessages.ERROR_USER.getValue());
			parametersOK = false;
		}
		if(userSrcObj.getCert() == null){			
			LOGGER.error(ErrorMessages.ERROR_CERTIFICATE.getValue());
			parametersOK = false;
		}
		return parametersOK;
	}

}
